
#include "GraphWin.h"

using namespace std;


GraphWindow::GraphWindow(int X, int Y, int W, int H, const char *l) : Fl_Double_Window(X,Y,W,H,l)
{
  GraphWidth  = W; 
  GraphHeight = H; 
  nAxisX = GraphHeight; 
  nAxisY = 0; 
  nTicksX = 0; 
  nTicksY = 0; 
  xmin=ymin=0;
  xmax=10;
  ymax=10;
}
void GraphWindow::draw()
{
  // clear
  fl_rectf(0,0,GraphWidth,GraphHeight,0,0,0);

  // draw axis and ticks
  fl_color(FL_WHITE);
      
  // line style must follow color, (style,width(0=default),dashes)
  fl_line_style(FL_SOLID,0,NULL);

  fl_xyline(0,nAxisX,GraphWidth);
  fl_yxline(nAxisY,0,GraphHeight);

  if (nTicksX > 0)
    for (int tik = 0 ; tik < GraphWidth ; tik += nTicksX)
      fl_yxline(tik,nAxisX,nAxisX-5);
  if (nTicksY > 0)
    for (int tik = 0 ; tik < GraphHeight ; tik += nTicksY)
      fl_xyline(nAxisY,tik,nAxisY+5);

  // scale view
  fl_push_matrix();
  fl_scale(1,-1); // positive y increases up
  if (xmax==xmin) xmax += 10;
  if (ymax==ymin) ymax += 10;
  fl_scale(GraphWidth/(xmax-xmin),GraphHeight/(ymax-ymin));
  //fl_translate(nAxisY,-nAxisX);
  fl_translate(xmin,-ymax);

  CurveIter iter = curves.begin();
  CurveRef curvy;

  while (iter != curves.end())
    {
      curvy = (*iter);
      fl_color(curvy->r,curvy->g,curvy->b);
      
      // line style must follow color, (style,width(0=default),dashes)
      fl_line_style(curvy->nStyle,0,NULL);

      if (curvy->nDrawType == GRAPH_LINE)
	fl_begin_line();
  
      PtIter DPit = curvy->dataPoints.begin();
      while (DPit != curvy->dataPoints.end())
	{
	  if (curvy->nDrawType == GRAPH_BAR)
	    {
	      fl_begin_line();
	      fl_vertex((*DPit).x,0);
	    }

	  // (0,0) is in top left corner
	  fl_vertex((*DPit).x,(*DPit).y);

	  if (curvy->nDrawType == GRAPH_BAR)
	    fl_end_line();

	  ++DPit;
	}  

      if (curvy->nDrawType == GRAPH_LINE)
	fl_end_line();

      ++iter;
    }
  fl_pop_matrix();
  
  //Fl_Double_Window::draw();
}
