#ifndef _POLYPOINTCPP
#define _POLYPOINTCPP

// PolyPoint.cpp
// Bryan Blackford - bblackfo@ucsc.edu
// CMPS161 W10 - Program 2

#include <stdlib.h>
#include <math.h>

#include <string>
#include <iostream>
#include <vector>

using namespace std;

const double PI = 3.1415926535897932384626433832795;
const double PI_OVER_2 = 1.5707963267948966192313216916398;
const double PI_OVER_180 = 0.017453292519943295769236907684886;

// ORGANIZATION 
// . Point (xyz,pos/vel/accel,rgb,ID)
// v Poly  (ID,texture)
// v[*] Poly.PointContainer
// O PolyObject (transforms(translate,rot,world_rot,scale),ID,polyScreen)
// O[+] PolyObject.PolyContainer

typedef class PolyObject* PolyObjRef;
class PolyObject;
typedef class PolyObjContainer* PolyObjContRef;
class PolyObjContainer;

// NORMALS
// . Point.Normal (xyz) computed by poly normals
// + Poly.Normal (xyz) normalized cross product of 

typedef struct vertex3f {float x,y,z;};

//inline void Normalize3(double *v)
inline void Normalize3f(vertex3f *v)
{
  //   double len = sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
  double len = sqrt(v->x*v->x+v->y*v->y+v->z*v->z);
  if (len==0) return;
   v->x /= len;
   v->y /= len;
   v->z /= len;
}


// point structures
typedef struct Point
{
  bool bNormLoaded;
  vertex3f vert,norm;
  vertex3f texCoords;
  float x,y,z,w; // w used for lighting (0=directional,1=point)
  float vx,vy,vz;
  float ax,ay,az;
  float fRed,fGrn,fBlu,fAlf;
  struct Point *next,*prev,*traceRef;
  int ID;
  int index; // for indexed colors
} Point;
typedef struct Point* PointRef;

class PointContainer
{
public:
  PointRef first;
  int count;
public:
  PointContainer() {first = NULL; count = 0;}
  
  PointRef CreateNewPoint(float fx=0,float fy=0,float fz=0)
  {
    PointRef pNew = (Point*)malloc(sizeof(Point)); //new Point;
    pNew->prev = pNew->next = pNew->traceRef = NULL;
    pNew->vx=pNew->vy=pNew->vz=pNew->ax=pNew->ay=pNew->az = 0;
    pNew->vert.x = pNew->x = fx;
    pNew->vert.y = pNew->y = fy;
    pNew->vert.z = pNew->z = fz;
    pNew->norm.x = pNew->norm.y = pNew->norm.z = 0;
    pNew->texCoords.x = pNew->texCoords.y = pNew->texCoords.z = 0.0f;
    pNew->bNormLoaded = false;
    
    pNew->fRed=pNew->fGrn=pNew->fBlu = 0;
    pNew->fAlf = 1.0f;
      
    if (first == NULL)
      first = pNew;
    else
      {   
	Point *pCur = first;
	while (pCur->next != NULL)
	  {
	    pCur = pCur->next;
	  }
	pCur->next = pNew;
	pNew->prev = pCur;
      }
    count++;
    return pNew;
  }

  void SetColor(PointRef pt, float fR,float fG,float fB)
  {
    pt->fRed = fR;
    pt->fBlu = fB;
    pt->fGrn = fG;
  }
  
  PointRef CreateTracePoint(PointRef pt)
  {
    PointRef tracept = CreateNewPoint(pt->x,pt->y); // count++ in createnewpoint
    tracept->traceRef = pt;
    tracept->fRed = pt->fRed;
    tracept->fGrn = pt->fGrn;
    tracept->fBlu = pt->fBlu;
    return tracept;
  }
  
  void SetXYVelByDirection(PointRef pt,int dir)
  {
    float vel = sqrt(pow(pt->vx,2)+pow(pt->vy,2));
    
    //    pt->direction = dir;
    pt->vx = cos(dir)*vel;
    pt->vy = sin(dir)*vel;
    
  }

  void MoveAllPoints(bool bTrace = false)
  {
    PointRef pCur = first;
    while (pCur != NULL)
      {
	if (pCur->traceRef == NULL)
	  {   // not a trace
	    if (bTrace)
	      CreateTracePoint(pCur);
	    pCur->x += pCur->vx;
	    pCur->y += pCur->vy;
	    pCur->z += pCur->vz;
	    pCur->vx += pCur->ax;
	    pCur->vy += pCur->ay;
	    pCur->vz += pCur->az;
	  }
	pCur = pCur->next;
      }
  }
  
  void RemoveTraces(PointRef pt)
  {
    // remove traces linked to point		    
    PointRef traceCur = first;
    PointRef traceTemp;
    while (traceCur)
      {
	if (traceCur->traceRef == pt)
	  {
	    traceTemp = traceCur->next;
	    RemovePoint(traceCur);
	    traceCur = traceTemp;
	  }
	else
	  traceCur = traceCur->next;
      }
  }

  void RemovePoint(PointRef pt)
  {
    if (pt==first)
      { 
	first = pt->next;
	if (pt->next != NULL)
	  pt->next->prev = NULL;
      }
    else
      {
	if (pt->next != NULL) pt->next->prev = pt->prev;
	if (pt->prev != NULL) pt->prev->next = pt->next;
      }
    free(pt);
    count--;
  }

  PointRef GetID(int ID)
  {
    PointRef pCur = first;
    while (pCur != NULL)
      {
	if (pCur->ID == ID) return pCur;
	
	pCur = pCur->next;
      }
    return NULL;
  }

};
typedef class PointContainer* PointContRef;

enum POLYTYPE
  {
    POLYTYPE_DEFAULT=0,
    POLYTYPE_LINE=1,
    POLYTYPE_INDEXED,
    POLYTYPE_SUPERQUADRIC,
  };

class Poly
{
public:
  PointContainer points;

  POLYTYPE type;

  // Normal
  vertex3f normal;

  // Texture
  bool bTextureLoaded;
  unsigned int texture;
  int texWidth,texHeight;
  void *textureDataP;
  
  Poly *next,*prev;
  int ID;
  Poly::Poly() {ID=-1; next=prev=NULL; normal.x=normal.y=normal.z=0; bTextureLoaded=false; type = POLYTYPE_DEFAULT;}
  float GetX(PointRef pt) {return pt->x;}
  float GetY(PointRef pt) {return pt->y;}
  float GetZ(PointRef pt) {return pt->z;}
  // checks if ptCheck is in this poly by finding the number of intersections of a horizontal line through poly..odd number means pt inside
  bool PointInPolygon(PointRef ptCheck)
  {
    int NodesCross=0;
    PointRef pt = points.first;
    // point 2 will be point previous to pt, starting at the last point as pt is on first
    PointRef pt2 = points.first; 
    while (pt2->next != NULL) pt2 = pt2->next;
    
    while (pt != NULL) 
      {
	if ( ((GetY(pt) > ptCheck->y) != (GetY(pt2) > ptCheck->y)) &&
	     (ptCheck->x < ((GetX(pt2)-GetX(pt)) * (ptCheck->y-GetY(pt)) / (GetY(pt2)-GetY(pt)) + GetX(pt))) )
	  NodesCross++;// = !c;
	pt2=pt;
	pt = pt->next;
      }      
    return ((NodesCross%2)==1); 
  }
  PointRef AddPoint(float fx,float fy,float fz)
  {
    PointRef pt = points.CreateNewPoint(fx,fy,fz);

    return pt;
  }
  void ComputeSurfaceNormal()
  {
    // normal(); - finds a normal vector and normalizes it
    PointRef pts[3];

    pts[0] = points.first;
    if (pts[0]==NULL) return;
    pts[1] = pts[0]->next;
    if (pts[1]==NULL) return;
    pts[2] = pts[1]->next;
    if (pts[2]==NULL) return;

    vertex3f a, b;

    // calculate the vectors A and B
    // note that v[3] is defined with counterclockwise winding in mind
    // a
    a.x = pts[0]->vert.x - pts[1]->vert.x;
    a.y = pts[0]->vert.y - pts[1]->vert.y;
    a.z = pts[0]->vert.z - pts[1]->vert.z;
    // b
    b.x = pts[1]->vert.x - pts[2]->vert.x;
    b.y = pts[1]->vert.y - pts[2]->vert.y;
    b.z = pts[1]->vert.z - pts[2]->vert.z;

    // calculate the cross product and place the resulting vector
    // into the address specified by vertex_t *normal
    normal.x = (a.y * b.z) - (a.z * b.y);
    normal.y = (a.z * b.x) - (a.x * b.z);
    normal.z = (a.x * b.y) - (a.y * b.x);

    // normalize
    Normalize3f(&normal);
  }    
};
typedef class Poly* PolyRef;

class PolyContainer
{
public:
  int count;
  PolyRef first;
  PolyContainer() {count = 0; first = NULL;}
  PolyRef CreatePoly()
  {
    PolyRef pNew = new Poly();
    pNew->prev = pNew->next = NULL;
      
    if (first == NULL)
      first = pNew;
    else
      {   
	PolyRef pCur = first;
	while (pCur->next != NULL)
	  {
	    pCur = pCur->next;
	  }
	pCur->next = pNew;
	pNew->prev = pCur;
      }

    return pNew;

  }
  void DeletePoly(PolyRef pDel)
  {
    if (pDel==first)
      { 
	first = pDel->next;
	if (pDel->next != NULL)
	  pDel->next->prev = NULL;
      }
    else
      {
	if (pDel->next != NULL) pDel->next->prev = pDel->prev;
	if (pDel->prev != NULL) pDel->prev->next = pDel->next;
      }
    free(pDel);
    count--;
  }
  PolyRef GetID(int ID)
  {
    PolyRef pCur = first;
    while (pCur != NULL)
      {
	if (pCur->ID == ID)
	  return pCur;
	pCur = pCur->next;
      }
    return NULL;
  }
};
typedef class PolyContainer* PolyContRef;

enum XFORMTYPES 
  {
    XFORM_NONE=0,
    XFORM_TRANSLATE,
    XFORM_ROTATE_X,
    XFORM_ROTATE_Y,
    XFORM_ROTATE_Z,
    XFORM_TRANSLATE_V,//vertical
    XFORM_TRANSLATE_H,//horizontal
    XFORM_TRANSLATE_M,//forward/back Movement
    XFORM_SCALE,
    XFORM_SCALEX,
    XFORM_SCALEY,
    XFORM_SCALEZ,
    XFORM_WORLDROTX,
    XFORM_WORLDROTY,
    XFORM_WORLDROTZ,
    XFORM_TRACKBALL,
    XFORM_TRANSLATE_X,
    XFORM_TRANSLATE_Y,
    XFORM_TRANSLATE_Z
  };
enum GLUSHAPES
  {
    SHAPE_NONE=0,
    SHAPE_SPHERE,
    SHAPE_CYLINDER,
    SHAPE_CONE,
    SHAPE_CONE_UP,
    SHAPE_CUBE,
  };

enum OBJTYPE
  {
    OBJTYPE_DEFAULT=0,
    OBJTYPE_LINE=1
  };

struct Spatial {
  Spatial(float initx=0,float inity=0,float initz=0)
    :ix(initx),iy(inity),iz(initz)                   {reset();}
  void Spatial::reset() {
    x=ix;
    y=iy;
    z=iz;
    vx=vy=vz=ax=ay=az=0;
    sx=sy=sz=1;
    drx=dry=drz=dr=dg=db=da=rx=ry=rz=dsx=dsy=dsz=0;
    fRed=fGrn=fBlu=fAlf=1;
    bSetPos = false;
  }
  void Spatial::SetPos(float nx=0,float ny=0,float nz=0) {x=nx; y=ny; z=nz;}
  void Spatial::resetPos() {x=ix; y=iy; z=iz;}
  void Spatial::SetV(float nx=0,float ny=0,float nz=0) {vx=nx; vy=ny; vz=nz;}
  void Spatial::Stop() {vx=vy=vz=ax=ay=az=0;}
  float  x, y, z;
  float vx,vy,vz; // velocity
  float ax,ay,az; // acceleration
  float ix,iy,iz; //reset position

  bool bSetPos; // change abs pos
  float rx,ry,rz; // rotation
  float drx,dry,drz;
  float sx,sy,sz; //scale
  float dsx,dsy,dsz; //scale
  float fRed,fGrn,fBlu,fAlf;//r,g,b,a;
  float dr,dg,db,da; //color
  
  Spatial &operator+=(const Spatial &that) {
    x += that.x;
    y += that.y;
    z += that.z;
    vx += that.vx;
    vy += that.vy;
    vz += that.vz;
    ax += that.ax;
    ay += that.ay;
    az += that.az;
    return *this;
  }
  Spatial operator+(const Spatial &that) const {
    Spatial result = *this;
    result += that;
    return result;
  }
  Spatial &operator/=(const float divisor) {
    x /= divisor;
    y /= divisor;
    z /= divisor;
    vx /= divisor;
    vy /= divisor;
    vz /= divisor;
    ax /= divisor;
    ay /= divisor;
    az /= divisor;
    return *this;
  }
  Spatial operator/(const float that) const {
    Spatial result = *this;
    result /= that;
    return result;
  }
  Spatial &operator*=(const float multiple) {
    x *= multiple;
    y *= multiple;
    z *= multiple;
    vx *= multiple;
    vy *= multiple;
    vz *= multiple;
    ax *= multiple;
    ay *= multiple;
    az *= multiple;
    return *this;
  }
  Spatial operator*(const float that) const {
    Spatial result = *this;
    result *= that;
    return result;
  }
  void Change(const Spatial &change) {
    if (change.bSetPos){
      x = change.x;
      y = change.y;
      z = change.z;
    }
    x += change.vx;
    y += change.vy;
    z += change.vz;
    rx += change.drx;
    ry += change.dry;
    rz += change.drz;
    vx += change.ax;
    vy += change.ay;
    vz += change.az;
    sx += change.dsx;
    sy += change.dsy;
    sz += change.dsz;
    fRed += change.dr;
    fGrn += change.dg;
    fBlu += change.db;
    fAlf += change.da;

  }
};

class PolyObject
{
public:
  OBJTYPE type;
  PolyContainer polies;
  PointContainer vertices;
  PolyObjRef pChild;
  float w; // w used as for light direction
  Spatial spate; // spatial state
  //float rx,ry,rz; // object-relative rotational values 0=forward
  //float wrx,wry,wrz; // world-relative rotational values        
  //float sx,sy,sz;
  float fRadius;
  float fHeight;
  //float fRed,fGrn,fBlu,fAlf; // object color
  // lighting
  int   iShininess;
  float fSpecR,fSpecG,fSpecB,fSpecA;
  float fAmbR,fAmbG,fAmbB,fAmbA;
  // animation
  //float vx,vy,vz; // velocity
  //float ax,ay,az; // acceleration
  float mass;
  bool bFreeFall;
  int nRollingOn;

  // info
  string name;

  // screen texture
  Poly polyScreen;

  // link objects
  PolyObjRef objParent;

  // realtime transforms, stored in x/rx/sx
  float ShiftX,ShiftY,ShiftZ;
  float ShiftOriginX,ShiftOriginY,ShiftOriginZ;
  int ShiftXformType;
  double mRotation[16];
  bool bCamera,bMirror;
  int nGluShape;
  int ID;
  PolyObjRef next,prev;

  PolyObject::PolyObject()
  {
    // load identity matrix into mRotation
    for (int i=0;i<16;i++)
      mRotation[i] = 0.;
    mRotation[0] = 1.;
    mRotation[5] = 1.;
    mRotation[10] = 1.;
    mRotation[15] = 1.;

    type = OBJTYPE_DEFAULT;
    nGluShape = SHAPE_NONE;
    bCamera=bMirror=false;
    bFreeFall = true;
    nRollingOn = -1;
    //rx=ry=rz=wrx=wry=wrz=0; 
    //x=y=z=ax=ay=az=vx=vy=vz=0;
    mass=0;
    w=1; // point source light default
    //sx=sy=sz=1;
    // for glu shapes
    fRadius = 1.0f;
    fHeight = 1.0f;
    ShiftX=ShiftY=ShiftZ=ShiftOriginX=ShiftOriginY=ShiftOriginZ=0;
    ID=0; 
    ShiftXformType=XFORM_NONE;
    next=prev=NULL;
    objParent=NULL;
    pChild=NULL;
    //fRed=fGrn=fBlu=
    fSpecR=fSpecG=fSpecB=0.f;
    fSpecA = 1.f;
    iShininess = 1;
  }
  PolyObjRef CreateChild()
  {
    PolyObjRef pNew = new PolyObject();
    pNew->prev = pNew->next = NULL;
    pNew->objParent = this;

    if (pChild == NULL)
      pChild = pNew;
    else
      {   
	PolyObjRef pCur = pChild;
	while (pCur->next != NULL)
	  {
	    pCur = pCur->next;
	  }
	pCur->next = pNew;
	pNew->prev = pCur;
      }
    //count++;

    return pNew;
  }
  float PosX() 
  {
    if (ShiftXformType == XFORM_TRANSLATE || ShiftXformType == XFORM_TRANSLATE_X)
      return spate.x+ShiftX-ShiftOriginX;
    //    else if (ShiftXformType == XFORM_TRANSLATE_H)
    //return x+((ShiftX-ShiftOriginX)*(float)sin((RotY()+90)*PI_OVER_180) * 1.0f);
    //objCamera->z -= (float)cos((objCamera->RotY()+90)*PI_OVER_180) * 1.0f; 

    else
      return spate.x;
  }
  float PosY() 
  {
    if (ShiftXformType == XFORM_TRANSLATE)
      return spate.y+ShiftOriginY-ShiftY;
    if (ShiftXformType == XFORM_TRANSLATE_Y)
      return spate.y+ShiftX-ShiftOriginX;
    else
      return spate.y;
  }
  float PosZ() 
  {
    if (ShiftXformType == XFORM_TRANSLATE)
      return spate.z+ShiftZ-ShiftOriginZ;
    if (ShiftXformType == XFORM_TRANSLATE_Z)
      return spate.z+ShiftX-ShiftOriginX;
    //    else if (ShiftXformType == XFORM_TRANSLATE_H)
    //return z+((ShiftX-ShiftOriginX)*(float)cos((RotY()+90)*PI_OVER_180) * 1.0f);
    else
      return spate.z;
  }
  float ScaleX() 
  {
    float temp;
    if (ShiftXformType == XFORM_SCALEX || ShiftXformType == XFORM_SCALE)
      temp = spate.sx+(ShiftX-ShiftOriginX)/50.0f;
    else
      temp = spate.sx;
    if (temp <= 0.1) temp = 0.1;
    return temp;
  }
  float ScaleY() 
  {
    float temp;
    if (ShiftXformType == XFORM_SCALEY || ShiftXformType == XFORM_SCALE)
      temp = spate.sy+(ShiftX-ShiftOriginX)/50.0f;
    else
      temp = spate.sy;
    if (temp <= 0.1) temp = 0.1;
    return temp;
  }
  float ScaleZ() 
  {
    float temp;
    if (ShiftXformType == XFORM_SCALEZ || ShiftXformType == XFORM_SCALE)
      temp = spate.sz+(ShiftX-ShiftOriginX)/50.0f;
    else
      temp = spate.sz;
    if (temp <= 0.1) temp = 0.1;
    return temp;
  }
  
};


class PolyObjContainer
{
public:
  PolyObjRef first;
  int count;
  PolyObjContainer::PolyObjContainer(){count=0; first=NULL;}
  PolyObjRef CreateObj()
  {
    PolyObjRef pNew = new PolyObject();
    pNew->prev = pNew->next = NULL;
      
    if (first == NULL)
      first = pNew;
    else
      {   
	PolyObjRef pCur = first;
	while (pCur->next != NULL)
	  {
	    pCur = pCur->next;
	  }
	pCur->next = pNew;
	pNew->prev = pCur;
      }
    count++;

    return pNew;

  }
  void Delete(PolyObjRef pDel)
  {
    if (pDel==first)
      { 
	first = pDel->next;
	if (pDel->next != NULL)
	  pDel->next->prev = NULL;
      }
    else
      {
	if (pDel->next != NULL) pDel->next->prev = pDel->prev;
	if (pDel->prev != NULL) pDel->prev->next = pDel->next;
      }
    free(pDel);
    count--;
  }

  PolyObjRef GetID(int ID)
  {
    return GetID(ID,first);
  }

  PolyObjRef GetID(int ID,PolyObjRef startAt)
  {
    PolyObjRef pCur = startAt;
    while (pCur != NULL)
      {
	if (pCur->ID == ID)
	  return pCur;
	
	// recurse children
	PolyObjRef pObj = GetID(ID,pCur->pChild);
	if (pObj != NULL) return pObj;

	pCur = pCur->next;
      }
    return NULL;
  }  
  void ResetTransforms()
  {
    PolyObjRef pCur = first;
    while (pCur != NULL)
      {
	pCur->spate.reset();
	//pCur->rx=pCur->ry=pCur->rz=pCur->wrx=pCur->wry=pCur->wrz=0.0f;
	//pCur->sx=pCur->sy=pCur->sz = 1.0f;
	pCur->ShiftXformType = XFORM_NONE;
	pCur = pCur->next;
      }
    
  }
  float sign(float x)
  {
    if (x < 0) return -1.0f;
    if (x == 0) return 0.0f;
    return 1.0f;
  }
  float sqPowSin(float x,float p)
  {
    return sign(sin(x))*powf(fabs(sin(x)),p);
  }
  float sqPowCos(float x,float p)
  {
    return sign(cos(x))*powf(fabs(cos(x)),p);
  }
  float sq_cl(float e1,float e2,float e3){return    (e1-e2)/(e1+e2+e3);}
  float sq_cp(float e1,float e2,float e3){return 2.*(e2-e3)/(e1+e2+e3);}
  float sq_cs(float e1,float e2,float e3){return 3.*(e3)   /(e1+e2+e3);}
  
  void InitSuperQuadricObject(PolyObjRef obj,float e1,float e2,float e3)
  {
  // create object
    //  PolyObjRef obj = CreateObj();
  const float sharp = 3.0f;
  
  //calculate a and b
  float a,b;
  float cl = sq_cl(e1,e2,e3);
  float cp = sq_cp(e1,e2,e3);
  float cs = sq_cs(e1,e2,e3);
  int   qx;
  if (cl >= cp)
    {
      a = powf(1.-cp,sharp);
      b = powf(1.-cl,sharp);
      qx = 1;
    }
  else
    {
      a = powf(1.-cl,sharp);
      b = powf(1.-cp,sharp);
      qx = 0;
    }

  InitSuperQuadricObject(obj,a,b,qx);
  //return obj;
  }

  void InitSuperQuadricObject(PolyObjRef obj,float a,float b,int qx) {
    PolyRef poly;
    PointRef pt;
    float x,y,z;
    const double d = 0.1*PI;

    //for (float theta = -PI_OVER_2 ; (theta+d) <= PI_OVER_2 ; theta += d) {
    for (float phi = 0 ; (phi) <= PI ; phi += d) {
    
      // create polygon strip to go around quadric
      poly = obj->polies.CreatePoly();
      poly->type = POLYTYPE_SUPERQUADRIC;
      
      //for (float phi = -PI ; (phi-d) <= PI ; phi += d) {
      for (float theta = 0 ; (theta-d) <= 2*PI ; theta += d) {
	
	// calculate point
	if (qx == 0) { //qz
	  x = sqPowCos(theta,a)*sqPowSin(phi,b);
	  y = sqPowSin(theta,a)*sqPowSin(phi,b);
	  z = sqPowCos(phi,b);
	  /* from gamedev.net/wiki
	     x = sqPowCos(theta,a)*sqPowCos(phi,b);
	     y = sqPowCos(theta,a)*sqPowSin(phi,b);
	     z = sqPowSin(theta,a);
	  */
	}
	else { //qx
	  x =     sqPowCos(phi,b);
	  y = -1.*sqPowSin(theta,a)*sqPowSin(phi,b);
	  z =     sqPowCos(theta,a)*sqPowSin(phi,b);
	}
	// add points to poly
	pt = poly->AddPoint(x,y,z);
	// calculate normal at point
	/*
	  pt->norm.x = -(sqPowCos(theta,2.-a)*sqPowCos(phi,2.-b));
	  pt->norm.y = -(sqPowCos(theta,2.-a)*sqPowSin(phi,2.-b));
	  pt->norm.z = -(sqPowSin(theta,2.-a));
	*/
	if (qx == 0) {
	  pt->norm.x = (sqPowCos(theta,2.-a)*sqPowSin(phi,2.-b));
	  pt->norm.y = (sqPowSin(theta,2.-a)*sqPowSin(phi,2.-b));
	  pt->norm.z = (sqPowCos(theta,2.-a));
	}
	else {
	  pt->norm.x =  (sqPowCos(phi,2.-b));
	  pt->norm.y = -(sqPowSin(theta,2.-a)*sqPowSin(phi,2.-b));
	  pt->norm.z =  (sqPowCos(theta,2.-a)*sqPowSin(phi,2.-b));
	}
	
	pt->bNormLoaded = true;
	
	// add point from next row
	if (qx == 0) { //qz
	  x = sqPowCos(theta,a)*sqPowSin(phi+d,b);
	  y = sqPowSin(theta,a)*sqPowSin(phi+d,b);
	  z = sqPowCos(phi+d,b);	
	}
	else { //qx
	  x =     sqPowCos(phi+d,b);
	  y = -1.*sqPowSin(theta,a)*sqPowSin(phi+d,b);
	  z =     sqPowCos(theta,a)*sqPowSin(phi+d,b);
	}
	// add points to poly
	pt = poly->AddPoint(x,y,z);
	// calculate normal at point
	/*
	  pt->norm.x = -(sqPowCos(theta+d,2.-a)*sqPowCos(phi,2.-b));
	  pt->norm.y = -(sqPowCos(theta+d,2.-a)*sqPowSin(phi,2.-b));
	  pt->norm.z = -(sqPowSin(theta+d,2.-a));
	*/
	
	if (qx == 0) {
	  pt->norm.x = (sqPowCos(theta,2.-a)*sqPowSin(phi+d,2.-b));
	  pt->norm.y = (sqPowSin(theta,2.-a)*sqPowSin(phi+d,2.-b));
	  pt->norm.z = (sqPowCos(theta,2.-a));
	}
	else {
	  pt->norm.x =  (sqPowCos(phi+d,2.-b));
	  pt->norm.y = -(sqPowSin(theta,2.-a)*sqPowSin(phi+d,2.-b));
	  pt->norm.z =  (sqPowCos(theta,2.-a)*sqPowSin(phi+d,2.-b));
	}
	pt->bNormLoaded = true;
	

	/*
	  x = sign(cos(theta+d))*powf(fabs(cos(theta+d)),a)*(sign(cos(phi))*powf(fabs(cos(phi)),b));
	  y = sign(cos(theta+d))*powf(fabs(cos(theta+d)),a)*(sign(sin(phi))*powf(fabs(sin(phi)),b));
	  z = sign(sin(theta+d))*powf(fabs(sin(theta+d)),a);
	*/
	// add points to poly
	//      pt = poly->AddPoint(x,y,z);
	
	
      }
    }
    
  }
};

//PolyObjRef PolyObjContainer::CreateSuperQuadricObject(double a, double b)

#endif
