// Prog.cpp
// Bryan Blackford - bblackfo@ucsc.edu
// CMPS161 W10 - Program 3

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <string>
#include <iostream>
#include <sstream>
#include <vector>

#include <FL/glut.h>
#include <FL/glu.h>
#include <FL/Fl_Gl_Window.H>
#include <FL/Fl_Multiline_Output.H>
#include <FL/Fl_File_Chooser.H>

#include "PolyPoint.cpp"

#include "window_info.h"
#include "GraphWin.h"

#include "wiiuse.h"
#define MAX_WIIMOTES 1
wiimote** wiimotes;
void handle_event(struct wiimote_t* wm);

using namespace std;

#define STR_APP_NAME "bblackfo - CMPS161 Final - W10"

//#include "Bitmap.cpp"

const int WIDTH = 512;
const int HEIGHT = 512;
const int HALFWIDTH = 256;
const int HALFHEIGHT = 256;
const int NEARZ = 1000;
const int FARZ  = 1000;

const int ID_OBJ_START = 100;
const int ID_OBJ_END   = 999;

#define PI_OVER_180 0.017453292519943295769236907684886


const float Gravity = -9.8;

enum {PROJ_PERSPECTIVE=0,PROJ_ORTHO,PROJ_LAST};
int nProjType=PROJ_PERSPECTIVE; 

enum Shapes {CIRCLE,TRIANGLE,SQUARE, };

class CVector3
{
public:
  CVector3::CVector3(){x=y=z=0;}
  CVector3::CVector3(GLdouble fx,GLdouble fy,GLdouble fz){x=fx; y=fy; z=fz;}
  GLdouble x,y,z;
};



// debug vars/interface
Fl_Multiline_Output *Debox;
char chErr[256];

Fl_File_Chooser *chooser;

// option vars

// input vars
int nMouseX=0,nMouseY=0;
int nLastClickX = -1; int nLastClickY = -1;
GLdouble fLastClickX=0,fLastClickY=0;
GLdouble fMouseX=0,fMouseY=0;
GLdouble fLastMouseX=0,fLastMouseY=0;
bool bXdown=false,bYdown=false,bZdown=false,bAdown=false,bBdown=false,bCdown=false,bSdown=false,bHdown=false,bVdown=false,bPdown=false,bMdown=false,bRdown=false,bWdown=false,bGdown=false,bSpaceDown=false;

// camera vars
const float ZINIT = -200.0f;

bool bWireframe = false;
bool bDropShadow = true;
bool bLighting = true;

GLUquadric* SphereQuadric;

int n; 
float ffx=0,ffy=0,ffz=0;
char *arg1,*arg2;

// Function Prototypes
void animate(void*);
void GLinit();
CVector3 GetOGLPos(int x, int y);
CVector3 GetOrigin();
void DrawPolies(PolyContainer polyCont);
void DrawPoly(PolyRef poly);
int DrawPoints(PointContainer pointCont);
void DrawObject(PolyObjRef obj,bool bShadow=false);
void DrawShape(int shape,float scale,float fx,float fy,float fz=0,float fRed=1,float fGrn=1,float fBlu=1);
PolyObjRef LoadObjectDialog();
PolyObjRef LoadObjectFile(const char *coorfile, const char *polyfile);
void SetProjection();
void SetIdentityMatrixd(double *d);
void ResetCamera(PolyObjRef);
void LoadTransforms(PolyObjRef obj,bool bWorldCamera=false);
void LoadRotations(PolyObjRef,bool bCamera=false);
void Rotate(PolyObjRef obj,float rx,float ry,float rz);
void RenderStyle(int mode);

int LoadTexture(PolyRef poly, char *filename,int wrap=1);
int InitPortalPoly(PolyRef poly);

void Spice(PolyObjRef);

void DrawLines(vector<PolyObjRef> points);

//animation

void UpdateAcceleration(PolyObjRef obj,Spatial spate,Spatial &spate,float t);
Spatial SpatialChange(PolyObjRef obj, const Spatial init, 
		      const Spatial change,float t, float dt);



// Data Structures
PointContainer Points;
PolyContainer Polies;
PolyObjContainer Objects;
PolyContainer NodrawPolies;
int vertexCount,polyCount;
PolyObjContainer NodrawObjects;

// Pointers
PolyObjRef objEye,objDraggin,objLast,objSelected=NULL,objCamTex, cylIDF, cylID, cylClear, mirSph;
PolyObjRef objScreen;
PolyRef polyScreen;
PolyObjRef objLine;

vector<PolyObjRef> Lights;

// 161-Prog3 animation
// Line Control Points
vector<PolyObjRef> ControllerSpheres;
vector<PolyObjRef> AnimatedObjects;

struct animation {
  vector<Spatial> spatialRecord;
  int nFrame;
  bool bPlay,bLoop;
  PolyObjRef obj;
  animation(){nFrame=0; bPlay=bLoop=false; obj=NULL;}
};
vector<animation> animations;
int nAnimSelected = 0;
int nCurrentRecording=0;


//vector<Spatial> spatialRecord;
//int nFrame=-1;


// FLTK Control Window
#define STR_POS "Position"
#define STR_ROT "Rotation"
#define STR_SCALE "Scale"
#define STR_COLOR "Color"
enum {PROP_POSITION,PROP_ROTATION,PROP_SCALE,PROP_COLOR} MenuOrder;
typedef pair<string,int> strint;
enum {RENDER_WIRE,RENDER_SMOOTH,RENDER_FLAT} RenderMenuOrder;
vector<strint> RenderVec;
enum {NEW_MODEL,NEW_ANIMATION,NEW_SPHERE,NEW_CYLINDER,NEW_CONE} NewMenuOrder;
vector<strint> NewVec;

// graph window
GraphWindow *GraphWin;

// time
unsigned long aniTime = 0;

// window class
class MyWindow : public Fl_Glut_Window 
{
#define BUFFERSIZE 512
public:
  enum{DRAW_TEX,DRAW_SCREEN,DRAW_DEFAULT,DRAW_PICK};
  GLuint PickBuffer[BUFFERSIZE];
  int drawMode;
  void MyWindow::RecursiveCamTexDraw(PolyObjRef pObj);
  void draw();
  int handle(int);
  MyWindow(int,int y,int w,int h,const char *l=0);
  int MyWindow::PickScreenXY(int x,int y);
  int MyWindow::GetClosestPickedID(int hits);
  
} *GLwindow;

MyWindow::MyWindow(int x,int y,int w,int h,const char *l) : Fl_Glut_Window(x,y,w,h,l) {drawMode=DRAW_SCREEN;}

int MyWindow::PickScreenXY(int x,int y)
{
  int viewport[4];
  glSelectBuffer(BUFFERSIZE,PickBuffer);
  glRenderMode(GL_SELECT); // select mode does not draw, only stores name/extents
  // init name stack
  glInitNames();
  glPushName(0);

  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  // pick matrix of pixels around x,y
  glGetIntegerv(GL_VIEWPORT,viewport);
  gluPickMatrix(x,y,6.0,6.0,viewport);
  SetProjection();
  // draw
  drawMode=DRAW_PICK;
  draw();

  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glFlush();
  glMatrixMode(GL_MODELVIEW);
  return glRenderMode(GL_RENDER); // returns number of hits
}

int MyWindow::GetClosestPickedID(int hits)
{
  int choose = 0; //ID
  int depth = 0; // Store How Far Away It Is 
  bool bFirstPicked=false; // nothing picked yet
  
  // Loop Through All The Detected Hits
  for (int loop = 0; loop < hits; loop++)	
    {
      int id = PickBuffer[loop*4+3];
      if (id > 0)
	{
	  // If This Object Is Closer To Us Than The One We Have Selected
	  // or none selected yet
	  if (!bFirstPicked || PickBuffer[loop*4+1] < GLuint(depth))
	    {
	      // Select The Closer Object
	      choose = id;
	      depth = PickBuffer[loop*4+1];
	      bFirstPicked = true;
	    }
	}      
    }       
  return choose;
}

int MyWindow::handle(int event) 
{
  CVector3 pos(0,0,0);
  float fx,fy;
  int key;
  GLint hits;
  int pickID;
  int timePass;
  PointRef pt;
  CVector3 vec;
  GLint prevmode;
  PolyObjRef obj;
  PolyRef poly;
  vector<PolyObjRef>::iterator itor;
  Spatial change;
	    
  pos = GetOGLPos(Fl::event_x(),Fl::event_y());
  nMouseX = Fl::event_x(); nMouseY = Fl::event_y();
  fMouseX = pos.x; fMouseY = pos.y;
  
  switch(event) {
  case FL_PUSH:
    //... mouse down event ...
    //... position in Fl::event_x() and Fl::event_y()
    if (Fl::event_button3()) // Right mouse
      {	
	// object translation (xyz) and camera motion (HV)
	if (bXdown==true)
	  {
	    objDraggin = objSelected;
	    if (objDraggin)
	      objDraggin->ShiftXformType = XFORM_TRANSLATE_X;
	  }
	if (bYdown==true)
	  {
	    objDraggin = objSelected;
	    if (objDraggin)
	      objDraggin->ShiftXformType = XFORM_TRANSLATE_Y;
	  }
	if (bZdown==true)
	  {
	    objDraggin = objSelected;
	    if (objDraggin)
	      objDraggin->ShiftXformType = XFORM_TRANSLATE_Z;
	  }
	if (bHdown==true && Fl::event_shift()!=0)
	  {
	    objDraggin = objEye;
	    objDraggin->ShiftXformType = XFORM_TRANSLATE_H;
	  }
	if (bVdown==true && Fl::event_shift()!=0)
	  {
	    objDraggin = objEye;
	    objDraggin->ShiftXformType = XFORM_TRANSLATE_V;
	  }
	if (bSdown==true)
	  {
	    objDraggin = objSelected;
	    if (objDraggin)
	      objDraggin->ShiftXformType = XFORM_SCALE;
	  }
	if (bMdown==true && Fl::event_shift()!=0)
	  {
	    objDraggin = objEye;
	    if (objDraggin)
	      objDraggin->ShiftXformType = XFORM_TRANSLATE_M;
	  }
       
	if (objDraggin)
	  {
	    objDraggin->ShiftX=objDraggin->ShiftOriginX = nMouseX;
	    objDraggin->ShiftY=objDraggin->ShiftOriginY = nMouseY;
	    objDraggin->ShiftZ=objDraggin->ShiftOriginZ = 0;
	  }
		  
	nLastClickX = Fl::event_x(); nLastClickY = Fl::event_y();
	fLastClickX = fMouseX; fLastClickY = fMouseY;
	
      } // if push RMB
    else if (Fl::event_button2()) // middle mouse mmb
      { // scale model (s) and move camera on z (M)
	if (bSdown==true)
	  {
	    objDraggin = objSelected;
	    if (objDraggin)
	      objDraggin->ShiftXformType = XFORM_SCALE;
	  }
	if (bMdown==true && Fl::event_shift()!=0)
	  {
	    objDraggin = objEye;
	    objDraggin->ShiftXformType = XFORM_TRANSLATE_M;
	  }
	
	if (objDraggin)
	  {
	    objDraggin->ShiftX=objDraggin->ShiftOriginX = nMouseX;
	    objDraggin->ShiftY=objDraggin->ShiftOriginY = nMouseY;
	    objDraggin->ShiftZ=objDraggin->ShiftOriginZ = 0;
	  }

	nLastClickX = Fl::event_x(); nLastClickY = Fl::event_y();
	fLastClickX = fMouseX; fLastClickY = fMouseY;	    	
      } // if push MMB
    else if (Fl::event_button1()) // Left mouse LMB
      {
	if (bRdown==true && Fl::event_shift()!=0)
	  {
	    objDraggin = objEye;
	    objDraggin->ShiftXformType = XFORM_ROTATE_Z;
	  }
	if (bPdown==true && Fl::event_shift()!=0)
	  {
	    objDraggin = objEye;
	    objDraggin->ShiftXformType = XFORM_ROTATE_X;
	  }
	if (bYdown==true && Fl::event_shift()!=0)
	  {
	    objDraggin = objEye;
	    objDraggin->ShiftXformType = XFORM_ROTATE_Y;
	  }
	if (bAdown==true)
	  {
	    objDraggin = objSelected;
	    if (objDraggin)
	      objDraggin->ShiftXformType = XFORM_ROTATE_X;
	  }		
	if (bBdown==true)
	  {
	    objDraggin = objSelected;
	    if (objDraggin)
	      objDraggin->ShiftXformType = XFORM_ROTATE_Y;
	  }		
	if (bCdown==true)
	  {
	    objDraggin = objSelected;
	    if (objDraggin)
	      objDraggin->ShiftXformType = XFORM_ROTATE_Z;
	  }
	if (objDraggin == NULL)
	  {
	    //pick objects under cursor into PickBuffer
	    hits = PickScreenXY(nMouseX,HEIGHT-nMouseY);	
	    if (hits > 0)	
	      {
		pickID = GetClosestPickedID(hits);
		objSelected = objDraggin = Objects.GetID(pickID);
		if (objDraggin != NULL)
		  objDraggin->ShiftXformType = XFORM_TRANSLATE;
	      }
	  }
	if (objDraggin != NULL)
	  {	    
	    objDraggin->ShiftX=objDraggin->ShiftOriginX = nMouseX;
	    objDraggin->ShiftY=objDraggin->ShiftOriginY = nMouseY;
	    objDraggin->ShiftZ=objDraggin->ShiftOriginZ = 0;
	  }	    

	if (objLine)
	  {
	    //pt = objLine->vertices.CreateNewPoint(fMouseX,fMouseY,0);
	    //pt = objLine->polies.first->points.CreateNewPoint(fMouseX,fMouseY,0);
	    //pt->fGrn = 1.f; pt->fRed = 1.f; pt->fBlu = 1.f;
	  }

	//	pt = new Point();
	//pt->x = nMouseX; pt->y = nMouseY;
	//GraphWin->DataPoints.push_back(pt);
	//GraphWin->redraw();
      }
    fLastMouseX = nMouseX;
    fLastMouseY = nMouseY;

    return 1;
  case FL_MOUSEWHEEL:
    // shift z
    if (objDraggin != NULL)
      {
	objDraggin->ShiftZ += Fl::event_dy();
      }	
    if (bMdown==true && Fl::event_shift()!=0)
      {	/*
	if (Fl::event_dy() > 0)
	  {
	    objEye->spate.x -= (float)sin(objEye->RotY()*PI_OVER_180) * 5.0f;
	    objEye->spate.z -= (float)cos(objEye->RotY()*PI_OVER_180) * 5.0f; 
	  }
	else if (Fl::event_dy() < 0)
	  {
	    objEye->spate.x += (float)sin(objEye->RotY()*PI_OVER_180) * 5.0f;
	    objEye->spate.z += (float)cos(objEye->RotY()*PI_OVER_180) * 5.0f; 
	  }
	*/
      }
    return 1;
  case FL_MOVE:
    break;
  case FL_DRAG:
    //... mouse moved while down event ...
    if (objDraggin != NULL)
      {
	objDraggin->ShiftX = nMouseX;
	objDraggin->ShiftY = nMouseY;
      }

    if (butRec->value()==true && nAnimSelected >= 0){
      change.SetV(-(fLastMouseX-nMouseX),fLastMouseY-nMouseY);	
      animations[nAnimSelected].spatialRecord.push_back(change);
    }
    
    fLastMouseX = nMouseX;
    fLastMouseY = nMouseY;

    return 1;
  case FL_RELEASE:
    //... mouse up event ...
    if (objDraggin != NULL)
      {
	switch (objDraggin->ShiftXformType)
	  {
	  case XFORM_TRANSLATE_H:
	  case XFORM_TRANSLATE_V:
	  case XFORM_TRANSLATE_M:
	    glGetIntegerv(GL_MATRIX_MODE, &prevmode);
	    glMatrixMode(GL_MODELVIEW);
	    glPushMatrix();
	    glLoadIdentity();
	    LoadTransforms(objDraggin);
	    //CameraTransform();
	    vec = GetOrigin();
	    objEye->spate.x = -vec.x;
	    objEye->spate.y = -vec.y;
	    objEye->spate.z = -vec.z;
	    glPopMatrix();
	    glMatrixMode(prevmode);
	    break;
	  case XFORM_TRANSLATE:
	  case XFORM_TRANSLATE_X:
	  case XFORM_TRANSLATE_Y:
	  case XFORM_TRANSLATE_Z:
	    objDraggin->spate.x = objDraggin->PosX();
	    objDraggin->spate.y = objDraggin->PosY();
	    objDraggin->spate.z = objDraggin->PosZ();
	    break;
	  case XFORM_ROTATE_X:
	  case XFORM_ROTATE_Y:
	  case XFORM_ROTATE_Z:
 	    glGetIntegerv(GL_MATRIX_MODE, &prevmode);
	    glMatrixMode(GL_MODELVIEW);	    
	    glPushMatrix();
	    glLoadIdentity();
	    LoadRotations(objDraggin);
	    glGetDoublev(GL_MODELVIEW_MATRIX,objDraggin->mRotation);
	    glPopMatrix();
	    glMatrixMode(prevmode);
	    break;
	  case XFORM_SCALE:
	  case XFORM_SCALEX:
	  case XFORM_SCALEY:
	  case XFORM_SCALEZ:
	    //objDraggin->scale += (objDraggin->ShiftY-objDraggin->ShiftOriginY)/10.0f;
	    //objDraggin->scale = objDraggin->Scale();
	    objDraggin->spate.sx = objDraggin->ScaleX();
	    objDraggin->spate.sy = objDraggin->ScaleY();
	    objDraggin->spate.sz = objDraggin->ScaleZ();
	    break;
	  }

	objDraggin->ShiftXformType = XFORM_NONE;
	objDraggin->ShiftX=objDraggin->ShiftOriginX=0;
	objDraggin->ShiftY=objDraggin->ShiftOriginY=0;
	objDraggin->ShiftZ=objDraggin->ShiftOriginZ=0;
	objDraggin = NULL;
      }
    return 1;
  case FL_FOCUS :
  case FL_UNFOCUS :
    //... Return 1 if you want keyboard events, 0 otherwise
    return 1;

  case FL_KEYDOWN:
    //key = (int)*Fl::event_text();
    key = Fl::event_key();
    switch (key)
      {
      case ' ':
	// stop objects
	itor = AnimatedObjects.begin();
	while (itor != AnimatedObjects.end()) {
	  (*itor)->spate.Stop();
	  ++itor;
	}
	bSpaceDown = true;
	break;

	/*
      case '`':	
	objEye->x += (float)sin((objEye->RotY()+90)*PI_OVER_180) * 1.0f;
	objEye->z += (float)cos((objEye->RotY()+90)*PI_OVER_180) * 1.0f; 
	break;
      
      case '4':	
	objEye->x -= (float)sin((objEye->RotY()+90)*PI_OVER_180) * 1.0f;
	objEye->z -= (float)cos((objEye->RotY()+90)*PI_OVER_180) * 1.0f; 
	break;
      
      case 'w':	
	objEye->x -= (float)sin(objEye->RotY()*PI_OVER_180) * 1.0f;
	objEye->z -= (float)cos(objEye->RotY()*PI_OVER_180) * 1.0f; 
	break;
      
      case '2':	
	objEye->x += (float)sin(objEye->RotY()*PI_OVER_180) * 1.0f;
	objEye->z += (float)cos(objEye->RotY()*PI_OVER_180) * 1.0f; 
	break;
	*/
//      case '1':	objEye->y += 1; break;
//      case '3':	objEye->y -= 1; break;

//      case 'q':	objEye->ry += 2; break;
//      case 'e':	objEye->ry -= 2; break;
      case 'x': bXdown = true; break;
      case 'y': bYdown = true; break;
      case 'z': bZdown = true; break;
      case 'a': 
	bAdown = true; 
	if (Fl::event_shift()!=0)
	  Spice(objSelected);
	break;
      case 'b': bBdown = true; break;
      case 'm': bMdown = true; break;
      case 'h': bHdown = true; break;
      case 'v': bVdown = true; break;
      case 'p': bPdown = true; break;
      case 's': bSdown = true; break;
      case 'w':
	bWdown = true; 
	if (Fl::event_shift()!=0)
	  RenderStyle(RENDER_WIRE);
	break;	
      case 'g':
	bGdown = true; 
	if (Fl::event_shift()!=0)
	  RenderStyle(RENDER_SMOOTH);       
	break;	
      case 'c': 
	bCdown = true; 
	if (Fl::event_shift()!=0)
	  RenderStyle(RENDER_FLAT);	      	    
	break;
      case 'r': 
	bRdown = true;
	if (Fl::event_shift()!=0)
	  {// RESET
	    Objects.ResetTransforms();
	    PolyObjRef pCur = Objects.first;
	    while (pCur)
	      {
		SetIdentityMatrixd(pCur->mRotation);
		pCur = pCur->next;
	      }
	    ResetCamera(objEye); 
	  }
	break; //'r'
      }
    break;

  case FL_KEYUP:
    //... keypress, key is in Fl::event_key(), ascii in Fl::event_text()
    //... Return 1 if you understand/use the keyboard event, 0 otherwise...
    if (Fl::event_key() == FL_Escape)
      exit(0);
    
    key = Fl::event_key();
    switch (key)
      {
      case ' ': bSpaceDown = false; break;
      case 'x': bXdown = false; break;
      case 'y': bYdown = false; break;
      case 'z': bZdown = false; break;
      case 'a': bAdown = false; break;
      case 'b': bBdown = false; break;
      case 'c': bCdown = false; break;
      case 'm': bMdown = false; break;
      case 'h': bHdown = false; break;
      case 'v': bVdown = false; break;
      case 'r': bRdown = false; break;
      case 's': bSdown = false; break;

      case ';':
	  obj = Objects.CreateObj();
	  objLast = obj;
	  obj->ID = ID_OBJ_START+Objects.count;
	  SetIdentityMatrixd(obj->mRotation);
	  
	  poly = obj->polies.CreatePoly();
	  
	  //poly->glTexImage2D(GL_TEXTURE_2D, 0, 3,WIDTH,WIDTH, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
	  //LoadTexture(poly,"texture.raw");

	  InitPortalPoly(poly);
	  pt = poly->points.CreateNewPoint(-64,-64,-10);
	  pt->fGrn = 1.f; pt->fRed = 1.f; pt->fBlu = 1.f; 
	  pt = poly->points.CreateNewPoint(64,-64,-10);
	  pt->fGrn = 1.f; pt->fRed = 1.f; pt->fBlu = 1.f; 
	  pt->texCoords.x = 1.f;
	  pt = poly->points.CreateNewPoint(64,64,-10);
	  pt->fGrn = 1.f; pt->fRed = 1.f; pt->fBlu = 1.f; 
	  pt->texCoords.x = 1.f; pt->texCoords.y = 1.f;
	  pt = poly->points.CreateNewPoint(-64,64,-10);
	  pt->fGrn = 1.f; pt->fRed = 1.f; pt->fBlu = 1.f; 
	  pt->texCoords.y = 1.f;
	  poly->normal.z = 1;
	  break;

      case '.':
	if (objSelected == NULL || objSelected == objEye)
	  objSelected = Objects.first;
	else if (objSelected != NULL)
	  objSelected = objSelected->next;
	break;
      case ',':
	if (objSelected == NULL || objSelected == objEye)
	  objSelected = Objects.first;
	else if (objSelected != NULL)
	  objSelected = objSelected->prev;
	break;
      case '/':
	objSelected = objEye;
	break;
      case 'd':
	bDropShadow = !bDropShadow;
	break;
      case ']': 
	objLast->spate.sx += .5; 
	objLast->spate.sy += .5;
	objLast->spate.sz += .5; 
	break;
      case '[': 
	objLast->spate.sx -= .5; 
	objLast->spate.sy -= .5;
	objLast->spate.sz -= .5; 
	break;
	//      case '=': objLast->ry += 36; break;
      case 'p': 
	bPdown = false;
	if (Fl::event_shift() == false)
	  {// cycle through projection types (perspective,orthographic,etc?)
	    nProjType++; 
	    if (nProjType>=PROJ_LAST) 
	      nProjType = 0;
	    // save matrixmode?
	    glMatrixMode(GL_PROJECTION);
	    glLoadIdentity();
	    
	    // Calculate the aspect ratio of the screen, and set up the
	    // projection matrix (i.e., update the camera)
	    SetProjection();
	  }
	break; //'p'
      }
    return 1;
  case FL_SHORTCUT:
    //... shortcut, key is in Fl::event_key(), ascii in Fl::event_text()
    //... Return 1 if you understand/use the shortcut event, 0 otherwise...
    return 1;
  default:
    // pass other events to the base class...
    return Fl_Glut_Window::handle(event);
  }
}


void MyWindow::RecursiveCamTexDraw(PolyObjRef pObj)
{
  while (pObj)
    {
      if (pObj->bCamera == true && pObj->polyScreen.bTextureLoaded == true)
	{
	  objCamTex = pObj; 
	  draw();
	}
      RecursiveCamTexDraw(pObj->pChild);
      pObj=pObj->next;
    }
}

void MyWindow::draw()
{  
  // the valid() property may be used to avoid reinitializing your
  // GL transformation for each redraw:
  if (!valid()) 
    {
      valid(1);
      glLoadIdentity();
      glViewport(0,0,WIDTH,HEIGHT);
    }
  
  PolyObjRef objCamera; // perspective we'll draw from

  if (drawMode==DRAW_DEFAULT)
    {
      // draw portal camera views
      drawMode = DRAW_TEX;
      // loop through objects
      RecursiveCamTexDraw(Objects.first);

      // draw screen
      drawMode = DRAW_SCREEN;
      draw();

      glutSwapBuffers();
      glutPostRedisplay();

      // return
      return;
    }
  if (drawMode==DRAW_TEX)
    {  
      //      glClearColor(.5f,.5f,.5f,0.5f);
      objCamera = objCamTex;
    }
  else if (drawMode==DRAW_SCREEN)
    {
      //glClearColor(0.f,0.f,0.f,1.f);
      objCamera = objEye;
    }
  else if (drawMode==DRAW_PICK)
    {
      objCamera = objEye;
    }

  if (drawMode != DRAW_PICK)
    {/*
      if (objCamera->bMirror)
	nProjType = PROJ_ORTHO;
      else
	nProjType = PROJ_PERSPECTIVE;
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      SetProjection();
     */
    }

  glMatrixMode(GL_MODELVIEW);

  // Clear the screen
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  
  glLoadIdentity();

  // camera/view transformations
  LoadTransforms(objCamera,(drawMode==DRAW_TEX));
  //CameraTransform();
  if (objCamera->bMirror)
    glScalef(1.0,1.0,-1.);

  // position lights
  if (drawMode==DRAW_PICK)
    glDisable(GL_LIGHTING);
  else
    {
      if (bLighting)
	glEnable(GL_LIGHTING);


      for (int i = 0 ; i < Lights.size() ; ++i)
	{
	  
	  float LightPos[]={Lights[i]->PosX(),Lights[i]->PosY(),Lights[i]->PosZ(),Lights[i]->w};
	  glLightfv(GL_LIGHT0+i,GL_POSITION,LightPos);

	  GLfloat color[] = {Lights[i]->spate.fRed,Lights[i]->spate.fGrn,Lights[i]->spate.fBlu,Lights[i]->spate.fAlf};

	  glLightfv(GL_LIGHT0+i, GL_DIFFUSE, color);
	  glLightfv(GL_LIGHT0+i, GL_AMBIENT, color);
	  //glLightfv(GL_LIGHT0+i, GL_SPECULAR,color);

	} 
    }

  //DrawPoints(Points);
  //  glDisable(GL_LIGHTING);
  //glDisable(GL_CULL_FACE);

  DrawPolies(Polies);

  //glEnable(GL_LIGHTING);
  //glEnable(GL_CULL_FACE);

  //DrawImage();

  DrawLines(ControllerSpheres);

  PolyObjRef obj = Objects.first;
  while (obj != NULL)
    {
      if (objCamera->bMirror == false || obj->bMirror == false)
	DrawObject(obj);

      // draw drop shadow object
      //glPushMatrix();
      //glTranslatef(0,-490,0);
      //glScalef(1,0.01,1);
      //DrawObject(obj,bDropShadow);
      //glPopMatrix();
      
      obj = obj->next;
    }

  if (objDraggin != NULL)
    {
      //DrawShape(TRIANGLE,25,objDraggin->PosX(),objDraggin->PosY(),objDraggin->PosZ(),0,0,1);
    }

  //glFlush();

  if (drawMode==DRAW_TEX)//objCamPoly)
    {
      /* Target texture */
      glBindTexture(GL_TEXTURE_2D, objCamera->polyScreen.texture);
      /* Copy screen to a texture */
      //glCopyTexImage2D(GL_TEXTURE_2D,0,GL_RGB,0,0,WIDTH,HEIGHT,0);
      glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, WIDTH, HEIGHT);
      //drawMode = DRAW_SCREEN;
      //draw();
      glBindTexture(GL_TEXTURE_2D,0);

      return;
    }

  if (drawMode==DRAW_SCREEN)
    {
      glutSwapBuffers();
      glutPostRedisplay();
    }
  
  // next draw will be default
  drawMode = DRAW_DEFAULT;  
  
}


void DrawPoint(float fR,float fG,float fB,float fx,float fy,float fz=0)
{
  glBegin(GL_POINTS);
  glColor3f(fR,fG,fB);
  glVertex3f(fx,fy,fz);
  glEnd();  
}

void DrawShape(int shape,float scale,float fx,float fy,float fz,float fRed,float fGrn,float fBlu)
{
  switch(shape)
    {
    case TRIANGLE:
      glBegin(GL_TRIANGLES);
      glColor3f(fRed,fGrn,fBlu);
      glVertex3f(fx-scale,fy-(scale/2),fz);
      glVertex3f(fx+scale,fy-(scale/2),fz);
      glVertex3f(fx      ,fy+(scale/2),fz);
      glEnd();
      break;
    case SQUARE:
      glBegin(GL_QUADS);
      glColor3f(fRed,fGrn,fBlu);
      glVertex3f(fx-(scale/2),fy-(scale/2),fz);
      glVertex3f(fx+(scale/2),fy-(scale/2),fz);
      glVertex3f(fx+(scale/2),fy+(scale/2),fz);
      glVertex3f(fx-(scale/2),fy+(scale/2),fz);
      glEnd();
      break;
    case CIRCLE:
      glBegin(GL_TRIANGLE_FAN);
      glColor3f(fRed,fGrn,fBlu);
      glVertex3f(fx,fy,fz);
      for (int i=0;i<=360;i+=9)
	glVertex3f(fx+(cosf(float(i))*scale),fy+(sinf(float(i))*scale),fz);
      glEnd();
      break;
    default:
      glBegin(GL_POINTS);      
      glColor3f(1,1,1);
      glVertex3f(fx,fy,fz);
      glEnd();
    };
}


int DrawPoints(PointContainer pointCont)
{
  int count = 0;
  PointRef pCur = pointCont.first;
  while (pCur != NULL)
    {
      glLoadName(pCur->ID); // assign ID

      DrawPoint(pCur->fRed,pCur->fGrn,pCur->fBlu,pCur->x,pCur->y,pCur->z);
      count++;
		  
      pCur = pCur->next;
    }
 
  return count;
}

void DrawLines(vector<PolyObjRef> points) {

  glBegin(GL_LINE_STRIP);
  vector<PolyObjRef>::iterator itor = points.begin();

  while (itor != points.end()) {
    //glColor4f(ptCur->fRed,ptCur->fGrn,ptCur->fBlu,ptCur->fAlf);
     
	  // with lighting
	  //GLfloat cyan[] = {0.f, .8f, .8f, 1.f};
    //	  GLfloat color[] = {ptCur->fRed,ptCur->fGrn,ptCur->fBlu,ptCur->fAlf};
    //	  glMaterialfv(GL_FRONT, GL_DIFFUSE,color);
    //	}

      
    glVertex3f((*itor)->PosX(),(*itor)->PosY(),(*itor)->PosZ());

    ++itor;
  }
  glEnd();
}

void DrawPolies(PolyContainer polyCont)
{
  PolyRef pCur = polyCont.first;

  while (pCur != NULL)
    {
      DrawPoly(pCur);
      pCur = pCur->next;
    }
}

void DrawPoly(PolyRef poly)
{
  PointRef ptCur = poly->points.first;
  PointRef ptnorm;

  //  glLoadName(poly->ID); // assign ID

  if (poly->bTextureLoaded==true)
    {
      glBindTexture(GL_TEXTURE_2D,poly->texture);
    }
  
  // draw polygon
  if (poly->type == POLYTYPE_LINE)  glBegin(GL_LINE_STRIP);
  else if (poly->type == POLYTYPE_SUPERQUADRIC) glBegin(GL_QUAD_STRIP);
  else if (poly->points.count == 1) glBegin(GL_POINTS);
  else if (poly->points.count == 2) glBegin(GL_LINES);
  else if (poly->points.count == 3) glBegin(GL_TRIANGLES);
  else if (bWireframe == false)     glBegin(GL_POLYGON); 
  else                              glBegin(GL_LINE_LOOP);
  
  // surface normal (backup if no vertex normals)
  glNormal3f(poly->normal.x,poly->normal.y,poly->normal.z);
  
  while (ptCur)
    {
      //if (poly->bTextureLoaded==false)
      if (poly->type != POLYTYPE_SUPERQUADRIC) // use object color
	{
	  glColor4f(ptCur->fRed,ptCur->fGrn,ptCur->fBlu,ptCur->fAlf);
     
	  // with lighting
	  //GLfloat cyan[] = {0.f, .8f, .8f, 1.f};
	  GLfloat color[] = {ptCur->fRed,ptCur->fGrn,ptCur->fBlu,ptCur->fAlf};
	  glMaterialfv(GL_FRONT, GL_DIFFUSE,color);
	}

      if (poly->bTextureLoaded==true)
	glTexCoord3f(ptCur->texCoords.x,ptCur->texCoords.y,ptCur->texCoords.z);

      if (ptCur->bNormLoaded==true)
	glNormal3f(ptCur->norm.x,ptCur->norm.y,ptCur->norm.z);
      
      glVertex3f(ptCur->x,ptCur->y,ptCur->z);
      
      // vertex normals
      //      if (ptCur->norm != NULL) // vertex normals

      ptCur = ptCur->next;
    }
  glEnd();
  
  if (poly->bTextureLoaded==true)
    glBindTexture(GL_TEXTURE_2D,0);// unbind texture

}

void DrawObject(PolyObjRef obj,bool bShadow)
{
  PolyRef pCur = obj->polies.first;

  // load transforms
  glPushMatrix();
    
  glTranslatef(obj->PosX(),obj->PosY(),obj->PosZ());
  glScalef(obj->ScaleX(),obj->ScaleY(),obj->ScaleZ());

  LoadRotations(obj);
  
  //if (obj == objSelected)
    //DrawShape(TRIANGLE,25,objSelected->PosX(),objSelected->PosY(),objSelected->PosZ(),0,0,1);

  if (obj->polyScreen.bTextureLoaded == true)
    {
      glBindTexture(GL_TEXTURE_2D,obj->polyScreen.texture);
    }
  
  //  glColor4f(obj->fRed,obj->fGrn,obj->fBlu,obj->fAlf);
  GLfloat color[]={obj->fSpecR,obj->fSpecG,obj->fSpecB,obj->fSpecA};
  glMaterialfv(GL_FRONT,GL_SPECULAR,color);
  glMateriali(GL_FRONT,GL_SHININESS,obj->iShininess);
  GLfloat color2[] = {obj->spate.fRed,obj->spate.fGrn,obj->spate.fBlu,obj->spate.fAlf};
  glMaterialfv(GL_FRONT, GL_DIFFUSE,color2);
  GLfloat color3[] = {obj->fAmbR,obj->fAmbG,obj->fAmbB,obj->fAmbA};
  glMaterialfv(GL_FRONT, GL_AMBIENT,color3);

  glColor4f(obj->spate.fRed,obj->spate.fGrn,obj->spate.fBlu,obj->spate.fAlf);
  
  glLoadName(obj->ID);

  // draw polygons
  if (obj->bCamera == false)
    while (pCur != NULL)
      {
	DrawPoly(pCur);
	pCur = pCur->next;
      }

  switch (obj->nGluShape)
    {
    case SHAPE_SPHERE:
      gluSphere(SphereQuadric,obj->fRadius,20,20); // quad,rad,xres,yres
      break;
    case SHAPE_CYLINDER:
      gluCylinder(SphereQuadric,obj->fRadius,obj->fRadius,obj->fHeight,20,10);
      break;
    case SHAPE_CONE:
      gluCylinder(SphereQuadric,0,obj->fRadius,obj->fHeight,20,10);
      break;
	case SHAPE_CONE_UP:
      gluCylinder(SphereQuadric,obj->fRadius,0,obj->fHeight,20,10);
	  break;
    }      

  if (obj->polyScreen.bTextureLoaded == true)
    glBindTexture(GL_TEXTURE_2D,0); // unbind

  // draw children
  PolyObjRef pCurChild = obj->pChild;
  while (pCurChild)
    {
      DrawObject(pCurChild);
      pCurChild = pCurChild->next;
    }

  glPopMatrix(); // return to origin (camera relative)  
}

enum {NO_COLLIDE=-1,};
int CollideMovingObjectWithLines(PolyObjRef obj,vector<PolyObjRef> &points,int startat = 0)
{
  int collided = NO_COLLIDE;
  int n=startat;

  while ((n+1) < points.size() && collided == NO_COLLIDE) {
    float x1 = points[n  ]->PosX();
    float x2 = points[n+1]->PosX();
    float y1 = points[n  ]->PosY();
    float y2 = points[n+1]->PosY();
    // sort by increasing x
    if (x1 > x2) {float t = x1; x1=x2; x2=t; t=y1; y1=y2; y2=t;}

    float x = obj->spate.x - x1;
    if (x > 0 && x < (x2-x1)) { // above the slope
	// get slope at x
	float m;
	if (x2-x1 == 0) m = PI_OVER_2;
	else            m = (y2-y1)/(x2-x1);
	float y = (m*x)+y1;
	//float theta = atan(m);
	
	// test if y value of ball is near y on the line
	y -= obj->spate.y;
	if (y < obj->fRadius && y > -obj->fRadius) {
	  collided = n;
	}
    }
    ++n;
  }

  return collided;
  
  
}

int main(int argc, char *argv[])
{
  
  GLwindow = new MyWindow(10,10,WIDTH,HEIGHT,STR_APP_NAME);
  GLwindow->mode(FL_RGB|FL_ALPHA|FL_DEPTH|FL_DOUBLE);
  // FL_ALPHA not supported over xming
  GLwindow->end();
  GLwindow->show();

  // Create Control Windows
  WindowInfo = make_window();
  WindowInfo->show();
  WindowInfo->position(15+WIDTH,30);  
  counter_x->lstep(10);
  counter_y->lstep(10);
  counter_z->lstep(10);
  ChoiceProperty->add(STR_POS);
  ChoiceProperty->add(STR_ROT);
  ChoiceProperty->add(STR_SCALE);
  ChoiceProperty->add(STR_COLOR);
  ChoiceProperty->value(0);
  ChoiceProperty->do_callback();

  RenderVec.push_back(strint("Wireframe",RENDER_WIRE));
  ChoiceRender->add(RenderVec.back().first.c_str());
  RenderVec.push_back(strint("Smooth",RENDER_SMOOTH));
  ChoiceRender->add(RenderVec.back().first.c_str());
  RenderVec.push_back(strint("Flat",RENDER_FLAT));
  ChoiceRender->add(RenderVec.back().first.c_str());

  //enum {NEW_MODEL,NEW_ANIMATION,NEW_SPHERE,NEW_CUBE} NewMenuOrder;
  NewVec.push_back(strint("Model",NEW_MODEL));
  ChoiceNew->add(NewVec.back().first.c_str());
  NewVec.push_back(strint("Animation",NEW_ANIMATION));
  ChoiceNew->add(NewVec.back().first.c_str());
  NewVec.push_back(strint("SPHERE",NEW_SPHERE));
  ChoiceNew->add(NewVec.back().first.c_str());
  NewVec.push_back(strint("CYLINDER",NEW_CYLINDER));
  ChoiceNew->add(NewVec.back().first.c_str());
  NewVec.push_back(strint("CONE",NEW_CONE));
  ChoiceNew->add(NewVec.back().first.c_str());
  
  // graph window
  GraphWin = new GraphWindow(10,HEIGHT+20,WIDTH,200,STR_APP_NAME);
  GraphWin->end();
  GraphWin->show(argc,argv);
  GraphWin->ymax = 1;
  GraphWin->ymin = -1;

  CurveRef curve = new Curve();
  curve->strName = "Rate";
  curve->r = 255; curve->g = 255; curve->b = 255;
  GraphWin->curves.push_back(curve);
  
  string strText = "Bryan Blackford\n";
  strText += STR_APP_NAME;
  strText += "\n\n";
  strText += "\nPress 'ESC' to quit.\n";
  strText += "(Enterprise not to scale)";
  outputbox->value(strText.c_str());

  objEye = NodrawObjects.CreateObj();
  objEye->name = "Eye";
  ResetCamera(objEye);
  
  sprintf(chErr,"none");
  
  // Create Objects
  GLinit();
  RenderStyle(1); //smooth
  PolyRef poly;
  PointRef pt;
  PolyObjRef obj;

// create sphere (lightsource)
  obj = Objects.CreateObj();
  obj->name = "Light1";
  objLast = obj;
  Lights.push_back(obj);
  ControllerSpheres.push_back(obj);
  obj->ID = ID_OBJ_START+Objects.count;
  SetIdentityMatrixd(obj->mRotation);
  obj->nGluShape = SHAPE_SPHERE;
  obj->fRadius = 10.;
  obj->fHeight = 1.;
  obj->fAmbR = 1.0f;
  obj->fAmbG = 1.0f;
  obj->fAmbB = 1.0f;
  obj->fAmbA = 1.f;
  obj->spate.fRed = 0.0;
  obj->spate.fGrn = 0.;
  obj->spate.fBlu = 1.;
  obj->spate.fAlf = 0.9;
  obj->spate.z = 0;
  obj->spate.y = -100;
  obj->spate.x = -200;
  obj->w = 0; // directional light from the camera

  obj = Objects.CreateObj();
  obj->name = "Light2";
  objLast = obj;
  Lights.push_back(obj);
  ControllerSpheres.push_back(obj);
  obj->ID = ID_OBJ_START+Objects.count;
  SetIdentityMatrixd(obj->mRotation);
  obj->nGluShape = SHAPE_SPHERE;
  obj->fRadius = 10.;
  obj->fHeight = 1.;
  obj->fAmbR = 1.0f;
  obj->fAmbG = 1.0f;
  obj->fAmbB = 1.0f;
  obj->fAmbA = 1.f;
  obj->spate.fRed = 0.0;
  obj->spate.fGrn = 1.0;
  obj->spate.fBlu = 0.0;
  obj->spate.fAlf = 0.9;
  obj->spate.SetPos(0,-100,0);
  obj->w = 0; 

  obj = Objects.CreateObj();
  obj->name = "Light3";
  objLast = obj;
  Lights.push_back(obj);
  ControllerSpheres.push_back(obj);
  obj->ID = ID_OBJ_START+Objects.count;
  SetIdentityMatrixd(obj->mRotation);
  obj->nGluShape = SHAPE_SPHERE;
  obj->fRadius = 10.;
  obj->fHeight = 1.;
  obj->fAmbR = 1.0f;
  obj->fAmbG = 1.0f;
  obj->fAmbB = 1.0f;
  obj->fAmbA = 1.0f;
  obj->spate.fRed = 1.0;
  obj->spate.fGrn = 0.0;
  obj->spate.fBlu = 0.0;
  obj->spate.fAlf = 0.9;
  obj->spate.SetPos(200,-100,0);
  obj->w = 0; 

  obj = Objects.CreateObj();
  obj->name = "Ball";
  objLast = obj;
  Lights.push_back(obj);
  //AnimatedObjects.push_back(obj);
  obj->ID = ID_OBJ_START+Objects.count;
  SetIdentityMatrixd(obj->mRotation);
  obj->nGluShape = SHAPE_SPHERE;
  obj->fRadius = 30.;
  obj->fHeight = 1.;
  obj->fAmbR = 0.0f;
  obj->fAmbG = 1.0f;
  obj->fAmbB = 0.0f;
  obj->fAmbA = 1.0f;
  obj->spate.fRed = 1.0;
  obj->spate.fGrn = 1.0;
  obj->spate.fBlu = 0.0;
  obj->spate.fAlf = 0.9;
  obj->spate.SetPos(0,200,0);
  obj->w = 0; 
  obj->mass = 1;

  obj = Objects.CreateObj();
  obj->name = "Ball2";
  objLast = obj;
  Lights.push_back(obj);
  //AnimatedObjects.push_back(obj);
  obj->ID = ID_OBJ_START+Objects.count;
  SetIdentityMatrixd(obj->mRotation);
  obj->nGluShape = SHAPE_SPHERE;
  obj->fRadius = 30.;
  obj->fHeight = 1.;
  obj->fAmbR = 0.0f;
  obj->fAmbG = 1.0f;
  obj->fAmbB = 0.0f;
  obj->fAmbA = 1.0f;
  obj->spate.fRed = 1.0;
  obj->spate.fGrn = 1.0;
  obj->spate.fBlu = 0.0;
  obj->spate.fAlf = 0.9;
  obj->spate.SetPos(0,200,0);
  obj->w = 0; 
  obj->mass = 1;
  

  nCurrentRecording=0;


  objSelected=obj=LoadObjectFile("data/enterprise.coor","data/enterprise.poly");
  obj->name = "Enterprise";
  obj->fSpecR = 1.f;
  obj->fSpecG = 1.f;
  obj->fSpecB = 1.f;
  obj->spate.fRed = 1.f;
  obj->spate.fGrn = 1.f;
  obj->spate.fBlu = 1.f;
  obj->spate.fAlf = 1.f;
  obj->fAmbR = 0.1f;
  obj->fAmbG = 0.1f;
  obj->fAmbB = 0.1f;
  obj->fAmbA = 1.f;
  obj->spate.SetPos(0,0,0);
  obj->spate.sx = .7;
  obj->spate.sy = .7;
  obj->spate.sz = .7;
  obj->iShininess = 96;
  animation anim;
  anim.obj = obj;
  anim.bPlay = false;
  anim.bLoop = false;
  animations.push_back(anim);
  AniList->add("anim0");

  
  //  chooser = new Fl_File_Chooser("~","poly and coor file (*)",Fl_File_Chooser::MULTI,"Choose a poly and coor file");
  chooser = new Fl_File_Chooser("~","ppm file (*.ppm)",Fl_File_Chooser::SINGLE,"Choose a gif file");

  wiimotes =  wiiuse_init(MAX_WIIMOTES);
  //found = 
  wiiuse_find(wiimotes, MAX_WIIMOTES, 50);
  //connected = 
  wiiuse_connect(wiimotes, MAX_WIIMOTES);
  wiiuse_set_leds(wiimotes[0], WIIMOTE_LED_1);

  // Start the animation timer
  Fl::add_timeout(0.1, animate);
    
  // Start the main loop
  glutMainLoop();

  return Fl::run();
}

void animate(void*)
{
  ++aniTime;
  Fl::repeat_timeout(0.04, animate);
    
  // debug text
  char ch[512];
  sprintf(ch,"ProjType: %d\nXdown: %s\nnMouse: %d %d\nfMouse: %f %f\nCamera: %f %f %f\nvertexCount: %d Poly: %d\nn: %d ffxyz: %f,%f,%f\nError: %s\nGL: %s\nGLerr: %d\nMaxLights: %d",nProjType,(bXdown?"y":"n"),nMouseX,nMouseY,fMouseX,fMouseY,objEye->PosX(),objEye->PosY(),objEye->PosZ(),vertexCount,polyCount,n,ffx,ffy,ffz,chErr, glGetString(GL_VERSION),glGetError(),GL_MAX_LIGHTS);
  
//Debox->value(ch);
  //outputbox->value(ch);

  if (objSelected != NULL)
    {
      SelName->value(objSelected->name.c_str());
      
      switch(ChoiceProperty->value())
	{
	case PROP_POSITION:
	  counter_x->value(objSelected->PosX());
	  counter_y->value(objSelected->PosY());
	  counter_z->value(objSelected->PosZ());       
	  break;
	case PROP_ROTATION:
	  counter_x->value(objSelected->spate.rx);
	  counter_y->value(objSelected->spate.ry);
	  counter_z->value(objSelected->spate.rz);       
	  break;
	case PROP_SCALE:
	  counter_x->value(objSelected->ScaleX());
	  counter_y->value(objSelected->ScaleY());
	  counter_z->value(objSelected->ScaleZ());       
	  break;	      
	case PROP_COLOR:
	  counter_x->value(objSelected->spate.fRed);
	  counter_y->value(objSelected->spate.fGrn);
	  counter_z->value(objSelected->spate.fBlu);       
	  break;	      
	  
	};
    }
  else 
    {
      SelName->value("-");
    }

  //counter_radius->value(AnimatedObjects[0]->fRadius);
  //counter_mass->value(AnimatedObjects[0]->mass);

  // animate objects 161p3
  static float t = 0;
  static float dt = .03;
  
  //for (int i = 0 ; !bSpaceDown && i < AnimatedObjects.size() ; ++i)
  if (0)
    {
      
      int i = 0;
      //rk2
      Spatial spate;
      Spatial k1 = SpatialChange(AnimatedObjects[i],AnimatedObjects[i]->spate,spate,t,0.0f);
      Spatial k2 = SpatialChange(AnimatedObjects[i],AnimatedObjects[i]->spate,k1,t + dt/2, dt/2 );

      Spatial change = ((k1+k2)/2.)*dt;
      AnimatedObjects[i]->spate.Change(change);
      //spatialRecord.push_back(change);
      t+=dt;

      /*
      ++i;
      if(spatialRecord.size() > 0){
	change = spatialRecord[0];
	spatialRecord.erase(spatialRecord.begin());
	AnimatedObjects[i]->spate.Change(change);
	}*/
    }
  else 
    {
      //spatialRecord.clear();
      //AnimatedObjects[1]->spate.reset();
    }
      
  for (int a = 0 ; a < animations.size() ; ++a){
    if (animations[a].nFrame >= animations[a].spatialRecord.size())
      {
	animations[a].nFrame = 0;
	if (animations[a].bLoop==false) {
	  animations[a].bPlay = false;
	  if (a==nAnimSelected)
	    butPlay->value(false);
	}
      }
    if (animations[a].bPlay){
      if (animations[a].nFrame < 0)
	  animations[a].nFrame = 0;
      animations[a].obj->spate.Change(
        animations[a].spatialRecord[animations[a].nFrame]);
      Rotate(animations[a].obj,animations[a].spatialRecord[animations[a].nFrame].drx,animations[a].spatialRecord[animations[a].nFrame].dry,animations[a].spatialRecord[animations[a].nFrame].drz);
      ++animations[a].nFrame;
      if (bSpaceDown) animations[a].obj->spate.reset();
	  }
    }

  if (wiiuse_poll(wiimotes, MAX_WIIMOTES)) {
    /*
     *	This happens if something happened on any wiimote.
     *	So go through each one and check if anything happened.
     */
    int i = 0;
    for (; i < MAX_WIIMOTES; ++i) {
      switch (wiimotes[i]->event) {
      case WIIUSE_EVENT:
	/* a generic event occured */
	handle_event(wiimotes[i]);
	break;
      }
    }
  }

  if (aniTime > GraphWin->xmax) {
    GraphWin->xmax = aniTime*2;
    // shift xmin as well for windowing
  }
  GraphWin->redraw();

  // Force a redisplay to render the new image
  glutPostRedisplay();
}

Spatial SpatialChange(PolyObjRef obj, const Spatial init, 
		      const Spatial change,float t, float dt) {
  Spatial spate;
  spate.x = init.x + change.vx*dt;
  spate.y = init.y + change.vy*dt;
  spate.z = init.z + change.vz*dt;
  spate.vx = init.vx + change.ax*dt;
  spate.vy = init.vy + change.ay*dt;
  spate.vz = init.vz + change.az*dt;

  Spatial out;
  out.vx = spate.vx;
  out.vy = spate.vy;
  out.vz = spate.vz;
  UpdateAcceleration(obj,spate,out,t+dt);
  return out;
}

void UpdateAcceleration(PolyObjRef obj,Spatial spate, Spatial &out, float t) { 
  int collide = 0;
  collide = CollideMovingObjectWithLines(obj,ControllerSpheres);
  if (collide == NO_COLLIDE){ // no collision
    //if (out.vy > MaxGravityVelocity)
      out.ay = Gravity;
      out.ax = 0;
      obj->bFreeFall = true;
      obj->nRollingOn = -1;
  }
  else {
    while (collide != NO_COLLIDE) {
      // obj is collided with line
      float x1 = ControllerSpheres[collide]->PosX();
      float x2 = ControllerSpheres[collide+1]->PosX();
      float y1 = ControllerSpheres[collide]->PosY();
      float y2 = ControllerSpheres[collide+1]->PosY();
      // sort by increasing x
      if (x1 > x2) {float t = x1; x1=x2; x2=t; t=y1; y1=y2; y2=t;}
      float m;
      if (x2-x1 == 0) m = PI_OVER_2;
      else            m = (y2-y1)/(x2-x1);
      float theta = atan(m);
      
      out.ay = Gravity*sinf(theta);
      out.ax = Gravity*cosf(theta);
      if (y2 <= y1) { // slope down right
	out.ay *= -1;
	out.ax *= -1;
      }
      
      float v = sqrtf((obj->spate.vx*obj->spate.vx)+
		      (obj->spate.vy*obj->spate.vy) );

      // if the obj has just collided, bounce or roll
      if (obj->bFreeFall) {
	obj->bFreeFall = false;
	if (v > 10) { // bounce
	  if (y2 > y1) // slope down left
	    v *= -1;
	  v *= 0.8;
	  obj->spate.vx = v*out.ay/Gravity; //-1/4;
	  obj->spate.vy = -v*out.ax/Gravity; // reducing bounce
	} else { // roll
	  obj->nRollingOn = collide;
	  obj->spate.vx = v*out.ax/Gravity;
	  obj->spate.vy = v*out.ay/Gravity;
	}
      }
      else if (obj->nRollingOn != -1) {
	if (obj->nRollingOn != collide) { 
	  // collided with new line, bounce
	  // check if angle between lines is less than 90
	  
	  //obj->spate.vx = v*out.ay/Gravity; //-1/4;
	  //obj->spate.vy = -v*out.ax/Gravity; // reducing bounce
	  
	}
      }
      else obj->nRollingOn = collide;
      collide = CollideMovingObjectWithLines(obj,ControllerSpheres,collide+1);
    }
  }
}

void callback_choicerender(Fl_Choice*, void*)
{  
  switch (RenderVec[ChoiceRender->value()].second)
    {
    case RENDER_WIRE:
      glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);	    
      bWireframe = true;
      break;
    case RENDER_SMOOTH:
      glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
      glShadeModel(GL_SMOOTH);
      gluQuadricNormals(SphereQuadric,GL_SMOOTH);
      bWireframe = false;
      break;
    case RENDER_FLAT:
      glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
      glShadeModel(GL_FLAT);
      gluQuadricNormals(SphereQuadric,GL_FLAT);
      bWireframe = false;
      break;
    }
}

void callback_choiceprop(Fl_Choice*, void*)
{
  double step,lstep;
  switch(ChoiceProperty->value())
    {
    case PROP_POSITION:
    case PROP_ROTATION:
      step = 0.1;
      lstep = 5;
      break;
    case PROP_SCALE:
    case PROP_COLOR:
      step = 0.05;
      lstep = 0.2;
      break;
    }

  counter_x->step(step);
  counter_x->lstep(lstep);
  counter_y->step(step);
  counter_y->lstep(lstep);
  counter_z->step(step);
  counter_z->lstep(lstep);

}

void callback_radius(Fl_Counter *o, void *v) 
{
  float value = (float)((Fl_Counter *)o)->value();
  AnimatedObjects[0]->fRadius = value;
}
void callback_mass(Fl_Counter *o, void *v) 
{
  float value = (float)((Fl_Counter *)o)->value();
  AnimatedObjects[0]->mass = value;
}
void callback_x(Fl_Counter *o, void *v) 
{
  if (objSelected == NULL) return;
  Spatial change;
  bool bSaveChange = true;
  float value = (float)((Fl_Counter *)o)->value();
  switch(ChoiceProperty->value())
    {
    case PROP_POSITION:
      change.SetV(-(objSelected->spate.x-value),0,0);
      objSelected->spate.x = value;
      break;
    case PROP_ROTATION:
      Rotate(objSelected,(value-objSelected->spate.rx),0,0);
      bSaveChange = false;
      break;
    case PROP_SCALE:
      change.dsx = value-objSelected->spate.sx;	
      objSelected->spate.sx = value;
      break;
    case PROP_COLOR:
      change.dr = value-objSelected->spate.fRed;
      objSelected->spate.fRed = value;
      break;
    };
  if (butRec->value()==true && nAnimSelected >= 0 && bSaveChange)
    animations[nAnimSelected].spatialRecord.push_back(change);
}

void callback_y(Fl_Counter *o, void *v) 
{
  if (objSelected == NULL) return;
  Spatial change;
  bool bSaveChange = true;
  float value = (float)((Fl_Counter *)o)->value();
  switch(ChoiceProperty->value())
    {
    case PROP_POSITION:
      change.SetV(0,-(objSelected->spate.y-value),0);
      objSelected->spate.y = value;
      break;
    case PROP_ROTATION:
      Rotate(objSelected,0,(value-objSelected->spate.ry),0);
      bSaveChange = false;
      break;
    case PROP_SCALE:
      change.dsy = value-objSelected->spate.sy;	
      objSelected->spate.sy = value;
      break;
    case PROP_COLOR:
      change.dg = value-objSelected->spate.fGrn;
      objSelected->spate.fGrn = value;
      break;
    };
  if (butRec->value()==true && nAnimSelected >= 0 && bSaveChange)
    animations[nAnimSelected].spatialRecord.push_back(change);
}

void callback_z(Fl_Counter *o, void *v) 
{
  if (objSelected == NULL) return;
  bool bSaveChange = true;
  Spatial change;
  float value = (float)((Fl_Counter *)o)->value();
  switch(ChoiceProperty->value())
    {
    case PROP_POSITION:
      change.SetV(0,0,-(objSelected->spate.z-value));
      objSelected->spate.z = value;
      break;
    case PROP_ROTATION:
      bSaveChange = false;
      Rotate(objSelected,0,0,(value-objSelected->spate.rz));
      break;
    case PROP_SCALE:
      change.dsz = value-objSelected->spate.sz;	
      objSelected->spate.sz = value;
      break;
    case PROP_COLOR:
      change.db = value-objSelected->spate.fBlu;
      objSelected->spate.fBlu = value;
      break;
    };
  if (butRec->value()==true && nAnimSelected >= 0 && bSaveChange)
    animations[nAnimSelected].spatialRecord.push_back(change);

}


// find xy pos on screen of a point in space
CVector3 GetScreenPos(GLdouble fx,GLdouble fy,GLdouble fz)
{
  GLint viewport[4];
  GLdouble modelview[16];
  GLdouble projection[16];
  GLdouble posX, posY, posZ;
  
  glGetDoublev( GL_MODELVIEW_MATRIX, modelview );
  glGetDoublev( GL_PROJECTION_MATRIX, projection );
  glGetIntegerv( GL_VIEWPORT, viewport );
  
  gluProject(fx,fy,fz,modelview,projection,viewport,&posX,&posY,&posZ);
  
  return CVector3(posX,posY,posZ);
}

CVector3 GetOGLPos(int x, int y)
{
  GLint viewport[4];
  GLdouble modelview[16];
  GLdouble projection[16];
  GLdouble winX, winY, winZ;
  GLdouble posX, posY, posZ;
  
  //glLoadIdentity();
  glGetDoublev( GL_MODELVIEW_MATRIX, modelview );
  glGetDoublev( GL_PROJECTION_MATRIX, projection );
  glGetIntegerv( GL_VIEWPORT, viewport );
  
  winX = (double)x;
  winY = (double)viewport[3] - (double)y;
  
  //glReadPixels( x, int(winY), 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &winZ );
  winZ = 1.0f;
  gluUnProject( winX, winY, winZ, modelview, projection, viewport, &posX, &posY, &posZ);
  
  return CVector3(posX, posY, posZ);
}

CVector3 GetOrigin()
{
  GLint viewport[4];
  GLdouble modelview[16];
  GLdouble projection[16];
  //GLdouble winX, winY, winZ;
  GLdouble posX, posY, posZ;
  
  //glLoadIdentity();
  glGetDoublev( GL_MODELVIEW_MATRIX, modelview );
  glGetDoublev( GL_PROJECTION_MATRIX, projection );
  glGetIntegerv( GL_VIEWPORT, viewport );
  
  //glReadPixels( x, int(winY), 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &winZ );
  //winZ = 1.0f;
  gluUnProject( HALFWIDTH, HALFHEIGHT, 0, modelview, projection, viewport, &posX, &posY, &posZ);
  
  return CVector3(posX, posY, posZ);
}


void GLinit()
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  // Calculate the aspect ratio of the screen, and set up the
  // projection matrix (i.e., update the camera)
  SetProjection();
  
  // Set up the OpenGL state
  glViewport(0,0,WIDTH,HEIGHT);
  glClearColor(0.1f,0.1f,0.1f,0.0f);
  glClearDepth(1.0f);
  //glClearStencil(0);
  //glDepthFunc(GL_LESS);
  glEnable(GL_DEPTH_TEST);
  //glDepthFunc(GL_LEQUAL);
  //glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
  glShadeModel(GL_SMOOTH);
  glEnable(GL_CULL_FACE);
  glLineWidth(2.);
  glEnable(GL_BLEND);
  //texture
  glEnable( GL_TEXTURE_2D );

  SphereQuadric = gluNewQuadric();
  gluQuadricNormals(SphereQuadric,GL_SMOOTH);
  gluQuadricTexture(SphereQuadric,GL_TRUE);
  //glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_SPHERE_MAP);
  //glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_SPHERE_MAP);
  gluQuadricOrientation(SphereQuadric, GLU_OUTSIDE);
  //gluSphere(SphereQuadric,10.,10,10); // quad,rad,xres,yres

  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  glHint(GL_POINT_SMOOTH, GL_NICEST);
  glHint(GL_LINE_SMOOTH, GL_NICEST);
  glHint(GL_POLYGON_SMOOTH, GL_NICEST);

  glEnable(GL_POINT_SMOOTH);
  glEnable(GL_LINE_SMOOTH);
  glEnable(GL_POLYGON_SMOOTH);

  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,GL_TRUE);

  if (bLighting)
    glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_LIGHT1);
  glEnable(GL_LIGHT2);
  glEnable(GL_LIGHT3);

  //glEnable(GL_COLOR_MATERIAL);
  //glColorMaterial(GL_FRONT_AND_BACK,GL_EMISSION);
  //glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);

  //GLfloat color[]={0.,0.,0.,1.};
  //  glMaterialfv(GL_FRONT,GL_SPECULAR,color);
  //glMateriali(GL_FRONT,GL_SHININESS,
//  GLfloat color2[] = {0.2,0.2,0.,1.};
  //glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,color2);

  glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}

PolyObjRef LoadObjectDialog()
{
  
  Fl_File_Chooser dlg("~","poly and coor file (*)",Fl_File_Chooser::MULTI,"Choose a poly and coor file");
  dlg.show();
  while (dlg.shown())
    Fl::wait();
  if (dlg.count() < 2) return NULL; // canceled
  char ch1[256];
  sprintf(ch1,"%s",dlg.value(1));
  char ch2[256];
  sprintf(ch2,"%s",dlg.value(2));
  return LoadObjectFile((const char*)ch1,(const char*)ch2);  
}

PolyObjRef LoadObjectFile(const char *coorfile, const char *polyfile)
{

  // read data from files
  FILE *inCoor,*inPoly;

  inCoor = fopen(coorfile,"r");
  if( inCoor==NULL )
    {
      printf("Unable to open file %s for reading\n", coorfile);
      exit(1);
    }

  inPoly = fopen(polyfile,"r");
  if (inPoly==NULL)
    {
      printf("Unable to open file %s for reading\n", polyfile);
      fclose(inCoor);
      exit(1);
    }
   
  PolyObjRef obj = Objects.CreateObj();
  objLast = obj;
  obj->ID = ID_OBJ_START+Objects.count;
  obj->name = polyfile;
  SetIdentityMatrixd(obj->mRotation);

  /* first line of coordinate input file contains vertex count */
  PointRef pt;
  //PolyRef vertpoly = new Poly(); //NodrawPolies.CreatePoly();
  fscanf(inCoor," %d",&vertexCount);
  for (int ii=1;ii < vertexCount;ii++)
    { // n==ii
      //fscanf(in,"%d",n);
      fscanf(inCoor," %d,%f,%f,%f",&n,&ffx,&ffy,&ffz);
      pt = obj->vertices.CreateNewPoint(ffx,ffy,ffz);
      //pt=vertpoly->AddPoint(ffx,ffy,ffz); 
      pt->fRed = 1; pt->fGrn = 1;
      pt->ID = n;
    }

  fclose(inCoor);

   
  fscanf(inPoly," %d",&polyCount);
  char chName[128],chNum[128]; int p1,p2,p3;
  PointRef newpt;
  PolyRef poly;
  char line[256];
  for (int ii = 1 ; ii < polyCount ; ii++)
    {
      if (fgets(line,sizeof(line),inPoly))
	{
	  int n;
	  if (n=sscanf(line," %s ",&chName) > 0)
	    {
	      char *ch = line+strlen(chName)+1;
	      
	      //line += n;
	      poly = obj->polies.CreatePoly();
	      poly->ID = obj->ID;
	      
	      while ((n=sscanf(ch," %s ",&chNum)) > 0 )// && (feof(line)) )
		{	  
		  ch+=strlen(chNum)+1;
		  
		  sscanf(chNum," %d ",&p1);
		  pt = obj->vertices.GetID(p1); 
		  if (pt)
		    {
		      newpt = poly->points.CreateNewPoint(pt->x,pt->y,pt->z);
		      newpt->fGrn=0.9; newpt->fRed=0.9; newpt->fBlu=0.9; 
		      newpt->ID=pt->ID;
		    }
		}
	      // compute surface normal and add to vertex normals
	      poly->ComputeSurfaceNormal();
	      // loop through vertices in polygon
	      PointRef polypt = poly->points.first;
	      while (polypt)
		{
		  pt = obj->vertices.GetID(polypt->ID);
		  // add surface normal to vertex normal
		  pt->norm.x += poly->normal.x;
		  pt->norm.y += poly->normal.y;
		  pt->norm.z += poly->normal.z;
		  polypt = polypt->next;
		}
	      
	    }
	}
    }

  // normalize vertex normals
  pt = obj->vertices.first;
  while (pt)
    {
      Normalize3f(&pt->norm);
      pt = pt->next;
    }

  // copy vertex normals from vertex list into polygon vertices
  poly = obj->polies.first;
  while (poly)
    {
      PointRef polypt = poly->points.first;
      while (polypt)
	{
	  pt = obj->vertices.GetID(polypt->ID);
	  if (pt)
	    {
	      polypt->norm = pt->norm;
	      polypt->bNormLoaded = true;
	    }
	  polypt = polypt->next;
	}      
      poly = poly->next;
    }

  fclose(inPoly);

  return obj;
}


void SetProjection()
{
  if (nProjType == PROJ_PERSPECTIVE)
    gluPerspective(60.0f,(GLfloat)WIDTH/(GLfloat)HEIGHT,0.1f,1500.0f);
  else if (nProjType == PROJ_ORTHO)
    glOrtho(-HALFWIDTH,HALFWIDTH,-HALFHEIGHT,HALFHEIGHT,-NEARZ,FARZ);
}

void SetIdentityMatrixd(double *d)
{
  GLint prevmode;
  glGetIntegerv(GL_MATRIX_MODE, &prevmode);
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();
  glGetDoublev(GL_MODELVIEW_MATRIX,d);
  glPopMatrix();
  glMatrixMode(prevmode);
}

void ResetCamera(PolyObjRef cam)
{
  cam->bCamera = true;
  SetIdentityMatrixd(cam->mRotation);
  cam->spate.reset();
  //cam->rx=cam->ry=cam->rz=0.0f;
  cam->spate.z = ZINIT;
}

void LoadTransforms(PolyObjRef obj, bool bWorldCamera)
{
  if (obj->ShiftXformType == XFORM_TRANSLATE_H)
    glTranslatef(obj->ShiftX-obj->ShiftOriginX,0,0);
  if (obj->ShiftXformType == XFORM_TRANSLATE_V)
    glTranslatef(0,obj->ShiftY-obj->ShiftOriginY,0);
  if (obj->ShiftXformType == XFORM_TRANSLATE_M)
    glTranslatef(0,0,obj->ShiftX-obj->ShiftOriginX);

  //glScalef(obj->sx,obj->sy,obj->sz);

  LoadRotations(obj,bWorldCamera);

  if (bWorldCamera)
    glTranslatef(-obj->PosX(),-obj->PosY(),-obj->PosZ());
  else
    glTranslatef(obj->PosX(),obj->PosY(),obj->PosZ());
}

void Rotate(PolyObjRef obj,float rx,float ry,float rz)
{
  int prevmode;
  glGetIntegerv(GL_MATRIX_MODE, &prevmode);
  glMatrixMode(GL_MODELVIEW);	    
  glPushMatrix();
  glLoadIdentity();
  LoadRotations(obj);
  glRotatef(rx,1.0,0.0,0.0);
  glRotatef(ry,0.0,1.0,0.0);
  glRotatef(rz,0.0,0.0,1.0);
  glGetDoublev(GL_MODELVIEW_MATRIX,obj->mRotation); //save rot
  glPopMatrix();
  glMatrixMode(prevmode);
  obj->spate.rx+=rx;
  obj->spate.ry+=ry;
  obj->spate.rz+=rz;
  if (butRec->value()==true && nAnimSelected >= 0) {
    Spatial change;
    change.drx = rx;
    change.dry = ry;
    change.drz = rz;
    animations[nAnimSelected].spatialRecord.push_back(change);
  }
}  

void LoadRotations(PolyObjRef obj,bool bCamera)
{  
  if (obj->bCamera == false)
    glMultMatrixd(obj->mRotation);
    
  GLfloat srot = 360.0f - (obj->ShiftX-obj->ShiftOriginX);
  if (obj->ShiftXformType == XFORM_ROTATE_X)
    glRotatef(srot,1.,0.,0.);
  if (obj->ShiftXformType == XFORM_ROTATE_Y)
    glRotatef(srot,0.,1.,0.);
  if (obj->ShiftXformType == XFORM_ROTATE_Z)
    glRotatef(srot,0.,0.,1.);

  if (obj->bCamera == true)
    glMultMatrixd(obj->mRotation);

  //GLfloat xrot = 360.0f - obj->RotX();
  //GLfloat yrot = 360.0f - obj->RotY();
  //GLfloat zrot = 360.0f - obj->RotZ();

  //glRotatef(xrot,1.f,0.f,0.f);
  //glRotatef(yrot,0.f,1.f,0.f);
  //glRotatef(zrot,0.f,0.f,1.f);
}

void Spice(PolyObjRef obj)
{
  PolyRef poly = obj->polies.first;
  while (poly)
    {
      PointRef pt = poly->points.first;
      while (pt)
	{
	  pt->fGrn=(rand()%100)/100.0;
	  pt->fRed=(rand()%100)/100.0;
	  pt->fBlu=(rand()%100)/100.0;
	  pt = pt->next;
	}
      poly = poly->next;
    }
}

int InitPortalPoly(PolyRef poly)
{
  void *data;
  int w = WIDTH;
  int h = HEIGHT;
  poly->texWidth=w;
  poly->texHeight=h;

  //data = malloc(w*h*3);
  //  glReadPixels( 0, 0, w, h, GL_RGB, GL_UNSIGNED_BYTE, &data ); // l,b

  //glEnable(GL_TEXTURE_2D);
  // allocate a texture name
  glGenTextures( 1, &poly->texture );
  
  //  GLwindow->draw();
  // select our current texture
  glBindTexture( GL_TEXTURE_2D, poly->texture );

  //  glCopyTexSubImage2D(GL_TEXTURE_2D,0,0,0,0,0,w,h);
  //glReadBuffer(GL_BACK);
  //glCopyTexImage2D(GL_TEXTURE_2D,0,GL_RGB,0,0,w,h,0);
  unsigned char* colorBits = new unsigned char[ WIDTH * HEIGHT * 3 ];  
  poly->textureDataP = colorBits;
  glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,WIDTH,HEIGHT,0, GL_RGB, 
	       GL_UNSIGNED_BYTE, colorBits);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

  //glReadPixels(0,0,WIDTH,HEIGHT,GL_RGB,GL_UNSIGNED_BYTE,colorBits);
  //clean up
  delete[] colorBits;

  int nErr = glGetError();
  if (nErr != GL_NO_ERROR)
    {
      switch(nErr)
	{
	case GL_INVALID_VALUE:
	  sprintf(chErr,"copytex: invalid_value",nErr);
	  break;
	case GL_INVALID_ENUM:
	  sprintf(chErr,"copytex: invalid enum",nErr);
	  break;
	case GL_INVALID_OPERATION:
	  sprintf(chErr,"copytex: invalid OP",nErr);
	  break;
	default:
	  sprintf(chErr,"copytex: err",nErr);
	  break;
	}
      return 0;
    }

  // select modulate to mix texture with color for shading
  //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
    
  // when texture area is small, bilinear filter the closest mipmap
  //glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
  //                     GL_LINEAR_MIPMAP_NEAREST );
  // when texture area is large, bilinear filter the first mipmap
  //glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  
  // if wrap is true, the texture wraps over at the edges (repeat)
  //       ... false, the texture ends at the edges (clamp)
  //glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,GL_CLAMP);
  //		   wrap ? GL_REPEAT : GL_CLAMP );
  //glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,GL_CLAMP);
  //		   wrap ? GL_REPEAT : GL_CLAMP );
  
  //  glGetTexImage(GL_TEXTURE_2D,3,GL_RGB,GL_UNSIGNED_BYTE,data)
  //glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, w, h, 0, GL_RGB,
  //	       GL_UNSIGNED_BYTE, poly->texture);
  
  // build our texture mipmaps
  //gluBuild2DMipmaps( GL_TEXTURE_2D, 3, poly->texWidth, poly->texHeight,
  //		     GL_RGB, GL_UNSIGNED_BYTE, data );
  
  // free buffer
  //  free( data );

  glBindTexture(GL_TEXTURE_2D,0);
  poly->bTextureLoaded=true;
  return 1;
    
  /*for (int i = 0; i < w ; i++)
    {
      for (int j = 0 ; j < h ; j++)
	{
	  gluUnProject( winX, winY, winZ, modelview, projection, viewport, &posX, &posY, &posZ);

      data[i] = r;
      data[i+1] = g;
      data[i+2] = b;
      }*/

}

int LoadTexture(PolyRef poly, char *filename,int wrap)
{
  void * data;
  FILE * file;

  // open texture data
  file = fopen( filename, "rb" );
  if ( file == NULL ) return 0;
  
  // allocate buffer
  poly->texWidth = 256;
  poly->texHeight = 256;
  data = malloc( poly->texWidth * poly->texHeight * 3 );

  // read texture data
  fread( data, poly->texWidth * poly->texHeight * 3, 1, file );
  fclose( file );
  
  // allocate a texture name
  glGenTextures( 1, &poly->texture );
  
  // select our current texture
  glBindTexture( GL_TEXTURE_2D, poly->texture );

  // select modulate to mix texture with color for shading
  glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
    
  // when texture area is small, bilinear filter the closest mipmap
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                     GL_LINEAR_MIPMAP_NEAREST );
  // when texture area is large, bilinear filter the first mipmap
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  
  // if wrap is true, the texture wraps over at the edges (repeat)
  //       ... false, the texture ends at the edges (clamp)
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,
		   wrap ? GL_REPEAT : GL_CLAMP );
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,
		   wrap ? GL_REPEAT : GL_CLAMP );
  
  // build our texture mipmaps
  gluBuild2DMipmaps( GL_TEXTURE_2D, 3, poly->texWidth, poly->texHeight,
		     GL_RGB, GL_UNSIGNED_BYTE, data );
  
  // free buffer
  free( data );
  glBindTexture(GL_TEXTURE_2D,0);// unbind texture
  
  return 1;
    
}

void RenderStyle(int mode)
{
  int i = 0;
  while (i < RenderVec.size() && RenderVec[i].second != mode)
    ++i;
  ChoiceRender->value(i);
  ChoiceRender->do_callback();	      
}

// Fl_Check_Button *checkLoop;
// Fl_Browser *AniList;
 void cb_anilist(Fl_Browser* o,void* v)
{
  nAnimSelected = o->value()-1;
  
  checkLoop->value(animations[nAnimSelected].bLoop);
  butPlay->value(animations[nAnimSelected].bPlay);
  objSelected = animations[nAnimSelected].obj;
  butRec->value(false);
}
 void cb_loop(Fl_Check_Button* o,void* v)
{
  animations[nAnimSelected].bLoop = checkLoop->value();
}
 void cb_play(Fl_Button* o,void* v)
{
  butRec->value(false);
  if (o->value()==true){
    animations[nAnimSelected].bPlay = true;
    animations[nAnimSelected].nFrame =0;
  }
  else{
    animations[nAnimSelected].bPlay = false;
  }
    
}
 void cb_record(Fl_Button* o,void* v)
{

}
void cb_new(Fl_Button* o,void* v)
{
  int nNew = ChoiceNew->value();
  switch (nNew) {
  case NEW_CYLINDER:
    objLast = Objects.CreateObj();
    objLast->name = "Cyl";
    //Lights.push_back(objLast);
    //ControllerSpheres.push_back(objLast);
    objLast->ID = ID_OBJ_START+Objects.count;
    SetIdentityMatrixd(objLast->mRotation);
    objLast->nGluShape = SHAPE_CYLINDER;
    objLast->fRadius = 3.;
    objLast->fHeight = 10.;
    break;
  case NEW_CONE:
    objLast = Objects.CreateObj();
    objLast->name = "Cone";
    //Lights.push_back(objLast);
    //ControllerSpheres.push_back(objLast);
    objLast->ID = ID_OBJ_START+Objects.count;
    SetIdentityMatrixd(objLast->mRotation);
    objLast->nGluShape = SHAPE_CONE;
    objLast->fRadius = 10.;
    objLast->fHeight = 10.;
    break;
  case NEW_SPHERE:
    objLast = Objects.CreateObj();
    objLast->name = "Sphere";
    //Lights.push_back(objLast);
    //ControllerSpheres.push_back(objLast);
    objLast->ID = ID_OBJ_START+Objects.count;
    SetIdentityMatrixd(objLast->mRotation);
    objLast->nGluShape = SHAPE_SPHERE;
    objLast->fRadius = 10.;
    objLast->fHeight = 1.;

    break;
  case NEW_MODEL:
    objLast = LoadObjectDialog();
    
    break;
  case NEW_ANIMATION:
    AniList->add("anim");
    animation anim;
    animations.push_back(anim);
    break;
  }
}
void cb_attach(Fl_Button* o,void* v)
{
  animations[nAnimSelected].obj = objSelected;  
}
 void cb_delete(Fl_Button* o,void* v)
{
  if (nAnimSelected >= 1){
    animations.erase(animations.begin()+nAnimSelected);
    nAnimSelected = 0;
    AniList->remove(nAnimSelected+1);
  }
}
 void cb_copy(Fl_Button* o,void* v)
{
  if (nAnimSelected >= 0){
    AniList->add("anim");
    animation anim;
    anim.bLoop = animations[nAnimSelected].bLoop;
    anim.spatialRecord = animations[nAnimSelected].spatialRecord;
    anim.obj = animations[nAnimSelected].obj;
    animations.push_back(anim);
  }
}
void cb_eye(Fl_Button*, void*)
{
  objSelected = objEye;
}
void cb_nextobj(Fl_Button*, void*)
{
  if (objSelected == NULL || objSelected == objEye)
    objSelected = Objects.first;
  else if (objSelected != NULL)
    objSelected = objSelected->next;
}
void cb_prevobj(Fl_Button*, void*)
{
  if (objSelected == NULL || objSelected == objEye)
    objSelected = Objects.first;
  else if (objSelected != NULL)
    objSelected = objSelected->prev;
}
void cb_resetpos(Fl_Button*, void*)
{
  if (objSelected){
    objSelected->spate.resetPos();
    //record
    if (butRec->value()==true && nAnimSelected >= 0){
      Spatial change(objSelected->spate.ix,objSelected->spate.iy,objSelected->spate.iz);
      change.bSetPos = true;
      animations[nAnimSelected].spatialRecord.push_back(change);
    }
  }
}
void cb_setresetpos(Fl_Button*, void*)
{
  if (objSelected){
    objSelected->spate.ix = objSelected->PosX();
    objSelected->spate.iy = objSelected->PosY();
    objSelected->spate.iz = objSelected->PosZ();
  }
}
void cb_adddelay(Fl_Button*, void*)
{
  if (butRec->value()==true && nAnimSelected >= 0){
    Spatial change;
    animations[nAnimSelected].spatialRecord.push_back(change);
  }
}
void cb_save(Fl_Button*, void*){}
void cb_load(Fl_Button*, void*){}
void cb_newchild(Fl_Button*, void*){

  if (objSelected){
    PolyObjRef obj = objSelected->CreateChild();
    ++Objects.count; //not incremented by createchild
    obj->ID = ID_OBJ_START+Objects.count;
    SetIdentityMatrixd(obj->mRotation);
    obj->spate.SetPos(10,10);

    int nNew = ChoiceNew->value();
    switch (nNew) {
    case NEW_CYLINDER:
      obj->name = objSelected->name+":Cyl";
      obj->nGluShape = SHAPE_CYLINDER;
      obj->fRadius = 3.;
      obj->fHeight = 10.;
      break;
    case NEW_CONE:
      obj->name = objSelected->name+":Cone";
      obj->nGluShape = SHAPE_CONE;
      obj->fRadius = 10.;
      obj->fHeight = 10.;
      break;
    case NEW_SPHERE:
      obj->name = objSelected->name+":Sphere";
      obj->nGluShape = SHAPE_SPHERE;
      obj->fRadius = 10.;
      obj->fHeight = 1.;
      break;
    }
  }
}
void cb_erase(Fl_Button*, void*){
  animations[nAnimSelected].spatialRecord.clear();
}

void cb_objlist(Fl_Browser*, void*){}


/**
 *	@brief Callback that handles an event.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *
 *	This function is called automatically by the wiiuse library when an
 *	event occurs on the specified wiimote.
 */
void handle_event(struct wiimote_t* wm) {
  static bool bBReleased = true;

  printf("\n\n--- EVENT [id %i] ---\n", wm->unid);

  /* if a button is pressed, report it */
  if (IS_PRESSED(wm, WIIMOTE_BUTTON_A))		printf("A pressed\n");
  if (IS_PRESSED(wm, WIIMOTE_BUTTON_B))		printf("B pressed\n");
  if (IS_PRESSED(wm, WIIMOTE_BUTTON_UP))		printf("UP pressed\n");
  if (IS_PRESSED(wm, WIIMOTE_BUTTON_DOWN))	printf("DOWN pressed\n");
  if (IS_PRESSED(wm, WIIMOTE_BUTTON_LEFT))     
    Rotate(objSelected,0,5,0);
  if (IS_PRESSED(wm, WIIMOTE_BUTTON_RIGHT))
    Rotate(objSelected,0,-5,0);
  if (IS_PRESSED(wm, WIIMOTE_BUTTON_MINUS))	printf("MINUS pressed\n");
  if (IS_PRESSED(wm, WIIMOTE_BUTTON_PLUS))	printf("PLUS pressed\n");
  if (IS_PRESSED(wm, WIIMOTE_BUTTON_ONE))		printf("ONE pressed\n");
  if (IS_PRESSED(wm, WIIMOTE_BUTTON_TWO))		printf("TWO pressed\n");
  if (IS_PRESSED(wm, WIIMOTE_BUTTON_HOME))	printf("HOME pressed\n");

  /*
   *	Pressing minus will tell the wiimote we are no longer interested in movement.
   *	This is useful because it saves battery power.
   */
  if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_MINUS))
    wiiuse_motion_sensing(wm, 0);

  /*
   *	Pressing plus will tell the wiimote we are interested in movement.
   */
  if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_PLUS))
    wiiuse_motion_sensing(wm, 1);

  /*
   *	Pressing B will toggle the rumble
   *
   *	if B is pressed but is not held, toggle the rumble
   */
  //  if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_B))
  //wiiuse_toggle_rumble(wm);

  if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_UP)) {
    wiiuse_set_leds(wiimotes[0], WIIMOTE_LED_3);
    wiiuse_set_ir(wm, 1);
  }
  if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_DOWN)) {
    wiiuse_set_leds(wiimotes[0], WIIMOTE_LED_3);
    wiiuse_set_ir(wm, 0);
  }

  if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_HOME)) {
    if (objSelected) {
      SetIdentityMatrixd(objSelected->mRotation);
      objSelected->spate.rx=objSelected->spate.ry=objSelected->spate.rz=0;
    }
  }

  /* if the accelerometer is turned on then print angles */
  float factor = 4.f;
  if (WIIUSE_USING_ACC(wm)) {
    if (IS_PRESSED(wm, WIIMOTE_BUTTON_A))
      if (objSelected == objEye)
	Rotate(objSelected,wm->orient.pitch/factor,0,wm->orient.roll/factor);
      else
	Rotate(objSelected,-wm->orient.roll/factor,0,wm->orient.pitch/factor);
    else if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_B))
      if (objSelected == objEye)
	Rotate(objSelected,wm->orient.a_pitch,0,wm->orient.a_roll);
      else
	Rotate(objSelected,-wm->orient.a_roll,0,wm->orient.a_pitch);

    

    GraphWin->curves[0]->dataPoints.push_back(XYPoint(aniTime,wm->orient.roll));
    if (wm->orient.roll > GraphWin->ymax) GraphWin->ymax = wm->orient.roll;
    if (wm->orient.roll < GraphWin->ymin) GraphWin->ymin = wm->orient.roll;

    printf("wiimote roll  = %f [%f]\n", wm->orient.roll, wm->orient.a_roll);
    printf("wiimote pitch = %f [%f]\n", wm->orient.pitch, wm->orient.a_pitch);
    printf("wiimote yaw   = %f\n", wm->orient.yaw);
  }

  /*
   *	If IR tracking is enabled then print the coordinates
   *	on the virtual screen that the wiimote is pointing to.
   *
   *	Also make sure that we see at least 1 dot.
   */
  if (WIIUSE_USING_IR(wm)) {
    int i = 0;

    /* go through each of the 4 possible IR sources */
    for (; i < 4; ++i) {
      /* check if the source is visible */
      if (wm->ir.dot[i].visible)
	printf("IR source %i: (%u, %u)\n", i, wm->ir.dot[i].x, wm->ir.dot[i].y);
    }

    printf("IR cursor: (%u, %u)\n", wm->ir.x, wm->ir.y);
    printf("IR z distance: %f\n", wm->ir.z);
  }
}
